@@ -49,11 +49,13 @@ app_state::app_state()
     search_root(current_root_path()),
     depth(-1), last(-1), next(-1),
     diff_format(unified_diff), diff_args_provided(false),
+    diff_show_encloser(true),
     execute(false), bind_address(""), bind_port(""),
     bind_stdio(false), use_transport_auth(true),
     missing(false), unknown(false),
     confdir(get_default_confdir()),
-    have_set_key_dir(false), no_files(false)
+    have_set_key_dir(false), no_files(false),
+    requested_help(false)
 {
   db.set_app(this);
   lua.set_app(this);
@@ -126,9 +128,9 @@ app_state::process_options()
 
     L(FL("branch name is '%s'") % branch_name());
 
-	  if (!options[key_option]().empty())
-		  internalize_rsa_keypair_id(options[key_option],
-					     signing_key);
+          if (!options[key_option]().empty())
+                  internalize_rsa_keypair_id(options[key_option],
+                                             signing_key);
   }
 }
 
@@ -152,7 +154,7 @@ app_state::create_workspace(system_path const & new_dir)
   go_to_workspace(new_dir);
 
   N(!directory_exists(bookkeeping_root),
-    F("monotone bookkeeping directory '%s' already exists in '%s'\n")
+    F("monotone bookkeeping directory '%s' already exists in '%s'")
     % bookkeeping_root % new_dir);
 
   L(FL("creating bookkeeping directory '%s' for workspace in '%s'")
@@ -269,7 +271,7 @@ app_state::set_date(utf8 const & d)
   catch (exception &e)
     {
       N(false, F("failed to parse date string '%s': %s")
-	% d % e.what());
+        % d % e.what());
     }
 }
 
