@@ -31,6 +31,7 @@
 #include "revision.hh"
 #include "transforms.hh"
 #include "vocab.hh"
+#include "globish.hh"
 
 using std::allocator;
 using std::basic_ios;
@@ -60,7 +61,7 @@ using std::vector;
 //   newline. Revision ids are printed in alphabetically sorted order.
 // Error conditions: If the branch does not exist, prints nothing.  (There are
 //   no heads.)
-AUTOMATE(heads)
+AUTOMATE(heads, N_("[BRANCH]"))
 {
   if (args.size() > 1)
     throw usage(help_name);
@@ -84,7 +85,7 @@ AUTOMATE(heads)
 //   newline. Revision ids are printed in alphabetically sorted order.
 // Error conditions: If any of the revisions do not exist, prints nothing to
 //   stdout, prints an error message to stderr, and exits with status 1.
-AUTOMATE(ancestors)
+AUTOMATE(ancestors, N_("REV1 [REV2 [REV3 [...]]]"))
 {
   if (args.size() == 0)
     throw usage(help_name);
@@ -131,7 +132,7 @@ AUTOMATE(ancestors)
 //   newline. Revision ids are printed in alphabetically sorted order.
 // Error conditions: If any of the revisions do not exist, prints nothing to
 //   stdout, prints an error message to stderr, and exits with status 1.
-AUTOMATE(descendents)
+AUTOMATE(descendents, N_("REV1 [REV2 [REV3 [...]]]"))
 {
   if (args.size() == 0)
     throw usage(help_name);
@@ -179,7 +180,7 @@ AUTOMATE(descendents)
 //   newline.  Revision ids are printed in alphabetically sorted order.
 // Error conditions: If any of the revisions do not exist, prints nothing to
 //   stdout, prints an error message to stderr, and exits with status 1.
-AUTOMATE(erase_ancestors)
+AUTOMATE(erase_ancestors, N_("[REV1 [REV2 [REV3 [...]]]]"))
 {
   set<revision_id> revs;
   for (vector<utf8>::const_iterator i = args.begin(); i != args.end(); ++i)
@@ -202,7 +203,7 @@ AUTOMATE(erase_ancestors)
 // Output format: A list of file names in alphabetically sorted order,
 //   or a list of attributes if a file name provided.
 // Error conditions: If the file name has no attributes, prints nothing.
-AUTOMATE(attributes)
+AUTOMATE(attributes, N_("[FILE]"))
 {
   if (args.size() > 1)
     throw usage(help_name);
@@ -252,7 +253,7 @@ AUTOMATE(attributes)
 //   newline.  Revisions are printed in topologically sorted order.
 // Error conditions: If any of the revisions do not exist, prints nothing to
 //   stdout, prints an error message to stderr, and exits with status 1.
-AUTOMATE(toposort)
+AUTOMATE(toposort, N_("[REV1 [REV2 [REV3 [...]]]]"))
 {
   set<revision_id> revs;
   for (vector<utf8>::const_iterator i = args.begin(); i != args.end(); ++i)
@@ -284,7 +285,7 @@ AUTOMATE(toposort)
 //   newline.  Revisions are printed in topologically sorted order.
 // Error conditions: If any of the revisions do not exist, prints nothing to
 //   stdout, prints an error message to stderr, and exits with status 1.
-AUTOMATE(ancestry_difference)
+AUTOMATE(ancestry_difference, N_("NEW_REV [OLD_REV1 [OLD_REV2 [...]]]"))
 {
   if (args.size() == 0)
     throw usage(help_name);
@@ -323,7 +324,7 @@ AUTOMATE(ancestry_difference)
 // Output format: A list of revision ids, in hexadecimal, each followed by a
 //   newline.  Revision ids are printed in alphabetically sorted order.
 // Error conditions: None.
-AUTOMATE(leaves)
+AUTOMATE(leaves, N_(""))
 {
   if (args.size() != 0)
     throw usage(help_name);
@@ -351,7 +352,7 @@ AUTOMATE(leaves)
 //   newline.  Revision ids are printed in alphabetically sorted order.
 // Error conditions: If the revision does not exist, prints nothing to stdout,
 //   prints an error message to stderr, and exits with status 1.
-AUTOMATE(parents)
+AUTOMATE(parents, N_("REV"))
 {
   if (args.size() != 1)
     throw usage(help_name);
@@ -375,7 +376,7 @@ AUTOMATE(parents)
 //   newline.  Revision ids are printed in alphabetically sorted order.
 // Error conditions: If the revision does not exist, prints nothing to stdout,
 //   prints an error message to stderr, and exits with status 1.
-AUTOMATE(children)
+AUTOMATE(children, N_("REV"))
 {
   if (args.size() != 1)
     throw usage(help_name);
@@ -409,7 +410,7 @@ AUTOMATE(children)
 //   The output as a whole is alphabetically sorted; additionally, the parents
 //   within each line are alphabetically sorted.
 // Error conditions: None.
-AUTOMATE(graph)
+AUTOMATE(graph, N_(""))
 {
   if (args.size() != 0)
     throw usage(help_name);
@@ -452,7 +453,7 @@ AUTOMATE(graph)
 // Output format: A list of revision ids, in hexadecimal, each followed by a
 //   newline. Revision ids are printed in alphabetically sorted order.
 // Error conditions: None.
-AUTOMATE(select)
+AUTOMATE(select, N_("SELECTOR"))
 {
   if (args.size() != 1)
     throw usage(help_name);
@@ -634,7 +635,7 @@ extract_added_file_paths(addition_map const & additions, path_set & paths)
 // Error conditions: If no workspace book keeping _MTN directory is found,
 //   prints an error message to stderr, and exits with status 1.
 
-AUTOMATE(inventory)
+AUTOMATE(inventory, N_(""))
 {
   if (args.size() != 0)
     throw usage(help_name);
@@ -672,7 +673,7 @@ AUTOMATE(inventory)
   classify_roster_paths(curr, unchanged, changed, missing, app);
   curr.extract_path_set(known);
 
-  restriction mask(app);
+  path_restriction mask(app);
   file_itemizer u(app, known, unknown, ignored, mask);
   walk_tree(file_path(), u);
 
@@ -816,7 +817,7 @@ AUTOMATE(inventory)
 //   the same type will be sorted by the filename they refer to.
 // Error conditions: If the revision specified is unknown or invalid
 // prints an error message to stderr and exits with status 1.
-AUTOMATE(get_revision)
+AUTOMATE(get_revision, N_("[REVID]"))
 {
   if (args.size() > 1)
     throw usage(help_name);
@@ -829,17 +830,17 @@ AUTOMATE(get_revision)
     {
       roster_t old_roster, new_roster;
       revision_id old_revision_id;
-      revision_set rev;
+      revision_t rev;
 
       app.require_workspace();
       get_base_and_current_roster_shape(old_roster, new_roster, nis, app);
       update_current_roster_from_filesystem(new_roster, app);
 
       get_revision_id(old_revision_id);
-      make_revision_set(old_revision_id, old_roster, new_roster, rev);
+      make_revision(old_revision_id, old_roster, new_roster, rev);
 
       calculate_ident(rev, ident);
-      write_revision_set(rev, dat);
+      write_revision(rev, dat);
     }
   else
     {
@@ -860,7 +861,7 @@ AUTOMATE(get_revision)
 //   on. This is the value stored in _MTN/revision
 // Error conditions: If no workspace book keeping _MTN directory is found,
 //   prints an error message to stderr, and exits with status 1.
-AUTOMATE(get_base_revision_id)
+AUTOMATE(get_base_revision_id, N_(""))
 {
   if (args.size() > 0)
     throw usage(help_name);
@@ -881,7 +882,7 @@ AUTOMATE(get_base_revision_id)
 //   files in the workspace.
 // Error conditions: If no workspace book keeping _MTN directory is found,
 //   prints an error message to stderr, and exits with status 1.
-AUTOMATE(get_current_revision_id)
+AUTOMATE(get_current_revision_id, N_(""))
 {
   if (args.size() > 0)
     throw usage(help_name);
@@ -890,7 +891,7 @@ AUTOMATE(get_current_revision_id)
 
   roster_t old_roster, new_roster;
   revision_id old_revision_id, new_revision_id;
-  revision_set rev;
+  revision_t rev;
   temp_node_id_source nis;
 
   app.require_workspace();
@@ -898,7 +899,7 @@ AUTOMATE(get_current_revision_id)
   update_current_roster_from_filesystem(new_roster, app);
 
   get_revision_id(old_revision_id);
-  make_revision_set(old_revision_id, old_roster, new_roster, rev);
+  make_revision(old_revision_id, old_roster, new_roster, rev);
 
   calculate_ident(rev, new_revision_id);
 
@@ -946,7 +947,7 @@ AUTOMATE(get_current_revision_id)
 //
 // Error conditions: If the revision ID specified is unknown or
 // invalid prints an error message to stderr and exits with status 1.
-AUTOMATE(get_manifest_of)
+AUTOMATE(get_manifest_of, N_("[REVID]"))
 {
   if (args.size() > 1)
     throw usage(help_name);
@@ -989,7 +990,7 @@ AUTOMATE(get_manifest_of)
 //
 // Error conditions: If the file id specified is unknown or invalid prints
 // an error message to stderr and exits with status 1.
-AUTOMATE(get_file)
+AUTOMATE(get_file, N_("FILEID"))
 {
   if (args.size() != 1)
     throw usage(help_name);
@@ -1015,7 +1016,7 @@ AUTOMATE(get_file)
 //
 // Error conditions: If the revision id specified is unknown or
 // invalid prints an error message to stderr and exits with status 1.
-AUTOMATE(packet_for_rdata)
+AUTOMATE(packet_for_rdata, N_("REVID"))
 {
   if (args.size() != 1)
     throw usage(help_name);
@@ -1041,7 +1042,7 @@ AUTOMATE(packet_for_rdata)
 //
 // Error conditions: If the revision id specified is unknown or
 // invalid prints an error message to stderr and exits with status 1.
-AUTOMATE(packets_for_certs)
+AUTOMATE(packets_for_certs, N_("REVID"))
 {
   if (args.size() != 1)
     throw usage(help_name);
@@ -1068,7 +1069,7 @@ AUTOMATE(packets_for_certs)
 //
 // Error conditions: If the file id specified is unknown or invalid
 // prints an error message to stderr and exits with status 1.
-AUTOMATE(packet_for_fdata)
+AUTOMATE(packet_for_fdata, N_("FILEID"))
 {
   if (args.size() != 1)
     throw usage(help_name);
@@ -1095,7 +1096,7 @@ AUTOMATE(packet_for_fdata)
 //
 // Error conditions: If any of the file ids specified are unknown or
 // invalid prints an error message to stderr and exits with status 1.
-AUTOMATE(packet_for_fdelta)
+AUTOMATE(packet_for_fdelta, N_("OLD_FILE NEW_FILE"))
 {
   if (args.size() != 2)
     throw usage(help_name);
@@ -1129,7 +1130,7 @@ AUTOMATE(packet_for_fdelta)
 // Error conditions: If any of the revisions do not exist, prints
 //   nothing to stdout, prints an error message to stderr, and exits
 //   with status 1.
-AUTOMATE(common_ancestors)
+AUTOMATE(common_ancestors, N_("REV1 [REV2 [REV3 [...]]]"))
 {
   if (args.size() == 0)
     throw usage(help_name);
@@ -1180,6 +1181,126 @@ AUTOMATE(common_ancestors)
       output << (*i).inner()() << endl;
 }
 
+// Name: branches
+// Arguments:
+//   None
+// Added in: 2.2
+// Purpose:
+//   Prints all branch certs present in the revision graph, that are not
+//   excluded by the lua hook 'ignore_branch'.
+// Output format:
+//   Zero or more lines, each the name of a branch. The lines are printed
+//   in alphabetically sorted order.
+// Error conditions:
+//   None.
+AUTOMATE(branches, N_(""))
+{
+  if (args.size() > 0)
+    throw usage(help_name);
+
+  vector<string> names;
+
+  app.db.get_branches(names);
+  sort(names.begin(), names.end());
+
+  for (vector<string>::const_iterator i = names.begin();
+       i != names.end(); ++i)
+    if (!app.lua.hook_ignore_branch(*i))
+      output << (*i) << endl;
+}
+
+// Name: tags
+// Arguments:
+//   A branch pattern (optional).
+// Added in: 2.2
+// Purpose:
+//   If a branch pattern is given, prints all tags that are attached to
+//   revisions on branches matched by the pattern; otherwise prints all tags
+//   of the revision graph.
+//
+//   If a branch name is ignored by means of the lua hook 'ignore_branch',
+//   it is neither printed, nor can it be matched by a pattern.
+// Output format:
+//   There is one basic_io stanza for each tag.
+//
+//   All stanzas are formatted by basic_io. Stanzas are separated
+//   by a blank line. Values will be escaped, '\' to '\\' and
+//   '"' to '\"'.
+//
+//   Each stanza has exactly the following four entries:
+//
+//   'tag'
+//         the value of the tag cert, i.e. the name of the tag
+//   'revision'
+//         the hexadecimal id of the revision the tag is attached to
+//   'signer'
+//         the name of the key used to sign the tag cert
+//   'branches'
+//         a (possibly empty) list of all branches the tagged revision is on
+//
+//   Stanzas are printed in arbitrary order.
+// Error conditions:
+//   A run-time exception is thrown for illegal patterns.
+AUTOMATE(tags, N_("[BRANCH_PATTERN]"))
+{
+  utf8 incl("*");
+  bool filtering(false);
+  
+  if (args.size() == 1) {
+    incl = idx(args, 0);
+    filtering = true;
+  }
+  else if (args.size() > 1)
+    throw usage(name);
+
+  globish_matcher match(incl, utf8());
+  basic_io::printer prt;
+  basic_io::stanza stz;
+  stz.push_str_pair(symbol("format_version"), "1");
+  prt.print_stanza(stz);
+  
+  vector<revision<cert> > tag_certs;
+  app.db.get_revision_certs(tag_cert_name, tag_certs);
+
+  for (vector<revision<cert> >::const_iterator i = tag_certs.begin();
+       i != tag_certs.end(); ++i) {
+
+    cert tagcert(i->inner());
+    vector<revision<cert> > branch_certs;
+    app.db.get_revision_certs(tagcert.ident, branch_cert_name, branch_certs);
+    
+    bool show(!filtering);
+    vector<string> branch_names;
+
+    for (vector<revision<cert> >::const_iterator j = branch_certs.begin();
+         j != branch_certs.end(); ++j) {
+
+      cert branchcert(j->inner());
+      cert_value branch;
+      decode_base64(branchcert.value, branch);
+      string branch_name(branch());
+      
+      if (app.lua.hook_ignore_branch(branch_name))
+        continue;
+      
+      if (!show && match(branch_name)) 
+        show = true;
+      branch_names.push_back(branch_name);
+    }
+
+    if (show) {
+      basic_io::stanza stz;
+      cert_value tag;
+      decode_base64(tagcert.value, tag);
+      stz.push_str_pair(symbol("tag"), tag());
+      stz.push_hex_pair(symbol("revision"), tagcert.ident);
+      stz.push_str_pair(symbol("signer"), tagcert.key());
+      stz.push_str_multi(symbol("branches"), branch_names);
+      prt.print_stanza(stz);
+    }
+  }
+  output.write(prt.buf.data(), prt.buf.size());
+}
 
 // Local Variables:
 // mode: C++
