@@ -1,13 +1,14 @@
 /*************************************************
-* Basic Interface Header File                    *
+* Botan Core Interface Header File               *
 * (C) 1999-2005 The Botan Project                *
 *************************************************/
 
 #include <botan/base.h>
 #include <botan/conf.h>
-#include <botan/exceptn.h>
 #include <botan/init.h>
 #include <botan/lookup.h>
 #include <botan/rng.h>
 #include <botan/version.h>
+#include <botan/bit_ops.h>
+#include <botan/parsing.h>
 
