@@ -11,6 +11,7 @@
 #include <iostream>
 #include <map>
 
+#include <boost/function.hpp>
 #include <boost/bind.hpp>
 
 #include "cmd.hh"
@@ -25,7 +26,8 @@ namespace automation {
   // guarantees about initialization order. So, use something we can
   // initialize ourselves.
   static map<string, automate * const> * automations;
-  automate::automate(string const &name)
+  automate::automate(string const &n, string const &p)
+   : name(n), params(p)
   {
     static bool first(true);
     if (first)
@@ -52,7 +54,7 @@ automate_command(utf8 cmd, vector<utf8> args,
     i->second->run(args, root_cmd_name, app, output);
 }
 
-static string const interface_version = "2.1";
+static string const interface_version = "2.2";
 
 // Name: interface_version
 // Arguments: none
@@ -63,7 +65,7 @@ static string const interface_version = "2.1";
 // Output format: "<decimal number>.<decimal number>\n".  Always matches
 //   "[0-9]+\.[0-9]+\n".
 // Error conditions: None.
-AUTOMATE(interface_version)
+AUTOMATE(interface_version, "")
 {
   if (args.size() != 0)
     throw usage(help_name);
@@ -204,7 +206,7 @@ static ssize_t automate_stdio_read(int d, void *buf, size_t nbytes)
   return rv;
 }
 
-AUTOMATE(stdio)
+AUTOMATE(stdio, "")
 {
   if (args.size() != 0)
     throw usage(help_name);
@@ -261,7 +263,12 @@ AUTOMATE(stdio)
                                       boost::ref(output),
                                       boost::ref(outpos),
                                       _1));
-          s.std::basic_ios<char, std::char_traits<char> >::rdbuf(&sb);
+          {
+            // Do not use s.std::basic_ios<...>::rdbuf here, 
+            // it confuses VC8.
+            using std::basic_ios;
+            s.basic_ios<char, std::char_traits<char> >::rdbuf(&sb);
+          }
           try
             {
               err=0;
@@ -291,36 +298,9 @@ AUTOMATE(stdio)
 }
 
 
-CMD(automate, N_("automation"),
-    N_("interface_version\n"
-       "heads [BRANCH]\n"
-       "ancestors REV1 [REV2 [REV3 [...]]]\n"
-       "attributes [FILE]\n"
-       "parents REV\n"
-       "descendents REV1 [REV2 [REV3 [...]]]\n"
-       "children REV\n"
-       "graph\n"
-       "erase_ancestors [REV1 [REV2 [REV3 [...]]]]\n"
-       "toposort [REV1 [REV2 [REV3 [...]]]]\n"
-       "ancestry_difference NEW_REV [OLD_REV1 [OLD_REV2 [...]]]\n"
-       "common_ancestors REV1 [REV2 [REV3 [...]]]\n"
-       "leaves\n"
-       "inventory\n"
-       "stdio\n"
-       "certs REV\n"
-       "select SELECTOR\n"
-       "get_file FILEID\n"
-       "get_manifest_of [REVID]\n"
-       "get_revision [REVID]\n"
-       "get_base_revision_id\n"
-       "get_current_revision_id\n"
-       "packet_for_rdata REVID\n"
-       "packets_for_certs REVID\n"
-       "packet_for_fdata FILEID\n"
-       "packet_for_fdelta OLD_FILE NEW_FILE\n"
-       "keys\n"),
-    N_("automation interface"),
-    OPT_NONE)
+CMD_PARAMS_FN(automate, N_("automation"),
+              N_("automation interface"),
+              OPT_NONE)
 {
   if (args.size() == 0)
     throw usage(name);
@@ -335,6 +315,21 @@ CMD(automate, N_("automation"),
   automate_command(cmd, cmd_args, name, app, std::cout);
 }
 
+std::string commands::cmd_automate::params()
+{
+  std::string out;
+  map<string, automation::automate * const>::const_iterator i;
+  for (i = automation::automations->begin();
+       i != automation::automations->end(); ++i)
+    {
+      char const * const p = commands::safe_gettext(i->second->params.c_str());
+      out += i->second->name + " " + p;
+      if (out[out.size()-1] != '\n')
+        out += "\n";
+    }
+  return out;
+}
+
 
 // Local Variables:
 // mode: C++
