@@ -138,7 +138,7 @@ ls_certs(string const & name, app_state & app, vector<utf8> const & args)
 
 static void
 ls_keys(string const & name, app_state & app, 
-	vector<utf8> const & args)
+        vector<utf8> const & args)
 {
   vector<rsa_keypair_id> pubs;
   vector<rsa_keypair_id> privkeys;
@@ -200,7 +200,7 @@ ls_keys(string const & name, app_state & app,
         }
       if (!all_in_db)
         cout << (F("(*) - only in %s/") 
-		 % app.keys.get_key_dir()) << "\n";
+                 % app.keys.get_key_dir()) << "\n";
       cout << "\n";
     }
 
@@ -246,7 +246,7 @@ ls_branches(string name, app_state & app, vector<utf8> const & args)
   sort(names.begin(), names.end());
   for (size_t i = 0; i < names.size(); ++i)
     if (match(idx(names, i)) 
-	&& !app.lua.hook_ignore_branch(idx(names, i)))
+        && !app.lua.hook_ignore_branch(idx(names, i)))
       cout << idx(names, i) << "\n";
 }
 
@@ -259,7 +259,7 @@ ls_epochs(string name, app_state & app, vector<utf8> const & args)
   if (args.size() == 0)
     {
       for (map<cert_value, epoch_data>::const_iterator 
-	     i = epochs.begin();
+             i = epochs.begin();
            i != epochs.end(); ++i)
         {
           cout << i->second << " " << i->first << "\n";
@@ -268,7 +268,7 @@ ls_epochs(string name, app_state & app, vector<utf8> const & args)
   else
     {
       for (vector<utf8>::const_iterator i = args.begin(); 
-	   i != args.end();
+           i != args.end();
            ++i)
         {
           map<cert_value, epoch_data>::const_iterator j = epochs.find(cert_value((*i)()));
@@ -332,8 +332,8 @@ ls_vars(string name, app_state & app, vector<utf8> const & args)
       external ext_domain, ext_name;
       externalize_var_domain(i->first.first, ext_domain);
       cout << ext_domain << ": " 
-	   << i->first.second << " " 
-	   << i->second << "\n";
+           << i->first.second << " " 
+           << i->second << "\n";
     }
 }
 
@@ -346,7 +346,9 @@ ls_known(app_state & app, vector<utf8> const & args)
   app.require_workspace();
   get_base_and_current_roster_shape(old_roster, new_roster, nis, app);
 
-  restriction mask(args, app.exclude_patterns, new_roster, app);
+  node_restriction mask(args_to_paths(args),
+                        args_to_paths(app.exclude_patterns),
+                        new_roster, app);
 
   node_map const & nodes = new_roster.all_nodes();
   for (node_map::const_iterator i = nodes.begin(); 
@@ -355,7 +357,7 @@ ls_known(app_state & app, vector<utf8> const & args)
       node_id nid = i->first;
 
       if (!new_roster.is_root(nid) 
-	  && mask.includes(new_roster, nid))
+          && mask.includes(new_roster, nid))
         {
           split_path sp;
           new_roster.get_name(nid, sp);
@@ -366,28 +368,36 @@ ls_known(app_state & app, vector<utf8> const & args)
 
 static void
 ls_unknown_or_ignored(app_state & app, bool want_ignored, 
-		      vector<utf8> const & args)
+                      vector<utf8> const & args)
 {
   app.require_workspace();
 
+  path_restriction mask(args_to_paths(args), args_to_paths(app.exclude_patterns), app);
   path_set unknown, ignored;
-  find_unknown_and_ignored(app, args, unknown, ignored);
+  find_unknown_and_ignored(app, mask, unknown, ignored);
 
   if (want_ignored)
     for (path_set::const_iterator i = ignored.begin(); 
-	 i != ignored.end(); ++i)
+         i != ignored.end(); ++i)
       cout << file_path(*i) << "\n";
   else
     for (path_set::const_iterator i = unknown.begin(); 
-	 i != unknown.end(); ++i)
+         i != unknown.end(); ++i)
       cout << file_path(*i) << "\n";
 }
 
 static void
 ls_missing(app_state & app, vector<utf8> const & args)
 {
+  temp_node_id_source nis;
+  roster_t current_roster_shape;
+  get_current_roster_shape(current_roster_shape, nis, app);
+  node_restriction mask(args_to_paths(args),
+                        args_to_paths(app.exclude_patterns),
+                        current_roster_shape, app);
+
   path_set missing;
-  find_missing(app, args, missing);
+  find_missing(current_roster_shape, mask, missing);
 
   for (path_set::const_iterator i = missing.begin(); 
        i != missing.end(); ++i)
@@ -409,11 +419,13 @@ ls_changed(app_state & app, vector<utf8> const & args)
 
   get_base_and_current_roster_shape(old_roster, new_roster, nis, app);
 
-  restriction mask(args, app.exclude_patterns, old_roster, new_roster, app);
+  node_restriction mask(args_to_paths(args),
+                        args_to_paths(app.exclude_patterns), 
+                        old_roster, new_roster, app);
 
   update_current_roster_from_filesystem(new_roster, mask, app);
   make_restricted_csets(old_roster, new_roster, 
-			included, excluded, mask);
+                        included, excluded, mask);
   check_restricted_cset(old_roster, included);
 
   set<node_id> nodes;
@@ -536,7 +548,7 @@ namespace
 //   private_location "keystore"
 //
 // Error conditions: None.
-AUTOMATE(keys)
+AUTOMATE(keys, "")
 {
   if (args.size() != 0)
     throw usage(help_name);
@@ -627,7 +639,7 @@ AUTOMATE(keys)
 // key, a warning message is printed to stderr. If the revision
 // specified is unknown or invalid prints an error message to stderr
 // and exits with status 1.
-AUTOMATE(certs)
+AUTOMATE(certs, N_("REV"))
 {
   if (args.size() != 1)
     throw usage(help_name);
