@@ -22,11 +22,11 @@
 #  - Browse in branches, revisions, diff files, view logs ...
 #
 # Needed tools:
-#  monotone 0.19 or compatible
+#  Monotone 0.19, 0.23, mtn 0.26 or compatible
 #  dialog (tested Version 0.9b)
-#  bash, sh, ash, dash
+#  bash, sh, dash
 #  less, vi or vim (use $VISUAL or $PAGER)
-#  cat, cut, echo, eval, head, sort, tail, wc, xargs ...
+#  cat, cut, echo, eval, head, sort, tail, tr, wc, xargs ...
 #
 # History:
 # 2005/5/5 Version 0.1.1 Henry@BigFoot.de
@@ -111,11 +111,35 @@
 # Temp files without tailing dot.
 # New option: Sort by Date/Time with up or down.
 # Fix: Parent of merge, if certs list from cache (ncolor).
+#
+# 2006-01-17 Version 0.1.14 Henry@BigFoot.de
+# One call for getting monotone version.
+# Warn before using "automate ancestor" on big database.
+#
+# 2006-01-21 Version 0.1.15 Henry@BigFoot.de
+# Fix: Author colors >8 wrong handled.
+# Faster brief selection menu, without changelog (do_log_brief_ancestors).
+# Skip merges from more as one person in brief selection.
+# Remove forbitten chars from changelog with tr. added backslash.
+#
+# 2006-01-23 Version 0.1.16 Henry@BigFoot.de
+# More posix shell syntax for dash.
+# do_head_sel: Prints error message from mt (it's first call with db file).
+#
+# 2006-04-01 Version 0.2.0 Henry@BigFoot.de
+# Handle bookkeeping directory MT and _MTN
+# Move checking bookkeeping directory behind versions check.
+#
+# 2006-04-10 Version 0.2.1 Henry@BigFoot.de
+# Binary is mtn, set by MTN, with fallback to 'monotone'.
 
 # Known Bugs / ToDo-List:
 # * better make "sed -n -e '1p'" for merge two different branches.
 
-VERSION="0.1.13"
+VERSION="0.2.1"
+
+# Binary
+MTN="mtn"
 
 # Save users settings
 # Default values, can overwrite on .mtbrowserc
@@ -162,7 +186,7 @@ FORMAT_LOG="F"
 FORMAT_COLOR="\\Z7\\Zb"
 
 # How get automate ancestors: I=interal function, A=automate ancestors,
-# L=Log brief
+# L=log brief with changelog, B=brief log
 ANCESTORS="L"
 
 # read saved settings
@@ -171,65 +195,6 @@ then
     . $CONFIGFILE
 fi
 
-# exist working copy?
-if [ -f MT/options ]
-then
-    # Read parameters from file
-    #  branch "mtbrowse"
-    #database "/home/hn/mtbrowse.db"
-    #     key ""
-
-    eval `sed -n -r \
-      -e 's/^[ ]*(branch) \"([^\"]+)\"$/\1=\2/p' \
-      -e 's/^[ ]*(database) \"([^\"]+)\"$/\1=\2/p' < MT/options`
-
-    if [ -n "$database" ]
-    then
-	DB=$database
-	BRANCH=$branch
-    fi
-fi
-
-
-# Simple program args supported
-if [ -n "$1" ]
-then
-    case $1 in
-      --version)
-	echo "mtbrowse $VERSION"
-	exit 0
-      ;;
-      --help|-h)
-	echo "mtbrowse [dbfile]"
-	exit 0
-      ;;
-      *)
-	# Databasefile from command line
-	DB="$1"
-	unset BRANCH
-
-	# MT change the options, if you continue with other DB here!
-	if [ -f MT/options ]
-	then
-
-	    if ! dialog --cr-wrap \
-		--title " *********** WARNING! ********** " \
-		--defaultno --colors --yesno "
-Your \Zb\Z1MT/options\Zn will be overwrite, if
-continue with different DB file or branch
-in exist working directory!
-
-YES confirm  /  NO abbort" 0 0
-	    then
-		echo "abbort"
-		exit 1
-	    fi
-	fi
-      ;;
-    esac
-fi
-
-
 # Clear cached files
 do_clear_cache()
 {
@@ -243,7 +208,7 @@ do_clear_on_exit()
 {
     rm -f $TEMPFILE.branches $TEMPFILE.ancestors $TEMPFILE.toposort \
       $TEMPFILE.action-select $TEMPFILE.menu $TEMPFILE.input \
-      $TEMPFILE.ncolor $TEMPFILE.tfc
+      $TEMPFILE.ncolor $TEMPFILE.tfc $TEMPFILE.error
 
     if [ "$CACHE" != "1" ]
     then
@@ -288,7 +253,7 @@ fill_date_key()
 {
     local in_file=$1
     local out_file=$2
-    local short_hash dat bra aut log lineno color count tfc
+    local hash short_hash dat bra aut log lineno color tfc
 
     line_count=`wc -l < $in_file`
     if [ "$line_count" -eq 0 ]
@@ -306,16 +271,16 @@ fill_date_key()
     do
 	if [ -n "$line_count" ]
 	then
-	    let lineno++
-	    echo "$(( 100*$lineno/line_count ))"
+	    lineno=$(( $lineno+1 ))
+	    echo "$(( 100*$lineno/$line_count ))"
 	else
 	    echo -n "." 1>&2
 	fi
 
 	short_hash=`echo $hash | cut -c 1-$HASH_TRIM`
 
-	# get certs of revision, cache it
-	monotone --db=$DB list certs $hash > $tfc
+	# get certs of revision, remove special chars, cache it
+	$MTN --db=$DB list certs $hash | tr '\\\t\042' '/ \047' > $tfc
 		
 	# Date format
 	case $FORMAT_DATE in
@@ -376,15 +341,15 @@ fill_date_key()
 
 	# Changelog format
 	case $FORMAT_LOG in
-	    F) # full     TAB here ----v
-		log=`sed -n -r -e "y/\"	/' /" -e \
+	    F) # full
+		log=`sed -n -r -e \
 		    '/^Name  : changelog/,+1s/Value : (.+)$/ \1/p' \
-		    < $tfc`
+		    < $tfc | sed -n -e '1p'`
 		;;
 	    S) # short
-		log=`sed -n -r -e "y/\"	/' /" -e \
+		log=`sed -n -r -e \
 		    '/^Name  : changelog/,+1s/Value : (.{1,20}).*$/ \1/p' \
-		    < $tfc`
+		    < $tfc | sed -n -e '1p'`
 		;;
 	esac
 
@@ -410,7 +375,8 @@ fill_date_key()
 		then
 		    color="\\Zb\\Z$color"
 		else
-		    color="\\Z$color"
+		    color8=$(( $color - 8 ))
+		    color="\\Z$color8"
 		fi
 	     else
 		color="$FORMAT_COLOR"
@@ -426,6 +392,117 @@ fill_date_key()
     rm $tfc
 }
 
+do_log_brief_ancestors()
+{
+    local out_file=$1
+    local hash short_hash dat bra aut lineno color
+
+    # Brief output
+    # e51dc90425c6371a176e87df294b47fcdba3f0bb Full Name <henry@bigfoot.de> 2005-11-20T20:31:34 mtbrowse
+
+    lineno=0
+    $MTN log --brief --last=$CERTS_MAX --revision=$HEAD --db=$DB |\
+    while read line
+    do
+	lineno=$(( $lineno+1 ))
+	# TODO: Why MT give more than "--last"?
+	if [ $lineno -le $CERTS_MAX ]
+	then
+	    echo "$(( 100*$lineno/$CERTS_MAX ))"
+	fi
+
+	set -- `echo "$line" | sed -n -r -e 's/^([^ ]+) (.+) ([0-9\-]{10})T([0-9:]{8}) (.+)$/\1 \3 \4 \5/p'`
+	author=`echo "$line" | sed -n -r -e 's/^([^ ]+) (.+) ([0-9\-]{10})T([0-9:]{8}) (.+)$/\2/p'`
+	hash=$1
+	date=$2
+	time=$3
+	branch="$4"
+
+	# Skip wrong formats, special case
+	# 37416b924fc25a48bba11ed8b2cd62e9dab7e637 First Name  <abc@gmail.com>,geecko@geek.com.au,third@domain.org 2006-01-20T08:21:37,2006-01-20T08:35:46,2006-01-20T08:35:33 net.venge.monotone
+
+	if [ -n "$hash" -a -n "$date" -a -n "$time" -a -n "$author" ]
+	then
+
+	    short_hash=`echo $hash | cut -c 1-$HASH_TRIM`
+
+	    # Date format
+	    case $FORMAT_DATE in
+	    F) # 2005-12-31T23:59:59
+		dat="${date}T$time "
+		;;
+	    L) # 2005-12-31 23:59
+		dat="`echo "$date $time" | cut -c 1-16` "
+		;;
+	    D) # 2005-12-31
+		dat=$date
+		;;
+	    S) # 12-31 23:59
+		dat=`echo "$date $time" | sed -n -r -e \
+		    '.{4}-(.+) (.{5}).+/\1 \2 /p'`
+		;;
+	    T) # 23:59:59
+		dat=$time
+		;;
+	    esac
+
+	    # Branch format
+	    case $FORMAT_BRANCH in
+	    F) # full
+		bra=$branch
+		;;
+	    S) # short
+		bra=`echo "$branch" | sed -n -r -e \
+		    '/.*\.([^\.]+)$/\1 /p' `
+		;;
+	    esac
+
+	    # Author format
+	    case $FORMAT_AUTHOR in
+	    F) # full
+		aut="$author"
+		;;
+	    S) # short
+		aut=`echo "$author " | sed -n -r -e \
+		    's/(.{1,10}).*[@ ].+$/\1/p'`
+		;;
+	    esac
+
+	    # TODO: Copied from fill_date_key
+	    # Author coloring?
+	    if [ -n "$FORMAT_COLOR" -a "$FORMAT_AUTHOR" != "N" ]
+	    then
+		if [ "$last_aut" != "$aut" ]
+		then
+		    # Automatic color by author?
+		    if [ "$FORMAT_COLOR" = "A" ]
+		    then
+			color=`grep -n "$aut" $TEMPFILE.ncolor | cut -d ':' -f 1`
+			if [ -z "$color" ]
+			then
+			    color=$(( `wc -l < $TEMPFILE.ncolor` % 16 + 1 ))
+			    echo "$aut" >> $TEMPFILE.ncolor
+			fi
+
+			if [ $color -le 8 ]
+			then
+			    color="\\Zb\\Z$color"
+			else
+			    color8=$(( $color - 8 ))
+			    color="\\Z$color8"
+			fi
+		    else
+			color="$FORMAT_COLOR"
+		    fi
+		    last_aut="$aut"
+		fi
+		echo "$short_hash \"$dat$bra\\ZR$color$aut\"" >> $out_file
+	    else
+		echo "$short_hash \"$dat$bra\\ZR$aut\"" >> $out_file
+	    fi
+	fi
+    done | dialog --gauge "$CERTS_MAX certs reading" 6 60
+}
 
 # Select a branch
 # Is parameter given: No user select, if branch known.
@@ -466,7 +543,7 @@ do_branch_sel()
     if [ ! -f $TEMPFILE.branches -o $DB -nt $TEMPFILE.branches \
 	-o "$CACHE" != "1" ]
     then
-	monotone --db=$DB list branches \
+	$MTN --db=$DB list branches \
 	 | sed -n -r -e 's/^(.+)$/\1\t-/p' > $TEMPFILE.branches \
 	 || exit 200
     fi
@@ -506,14 +583,19 @@ do_head_sel()
 	return
     fi
 
-    monotone --db=$DB automate heads $BRANCH > $TEMPFILE.heads 2>/dev/null
+    if ! $MTN --db=$DB automate heads $BRANCH > $TEMPFILE.heads 2>$TEMPFILE.error
+    then
+	cat $TEMPFILE.error
+	exit -1
+    fi
+
     # Only one head ?
     if [ `wc -l < $TEMPFILE.heads` -eq 1 -a -n "$1" ]
     then
 	HEAD=`head -n 1 < $TEMPFILE.heads`
     else
 	# List heads with author and date. Select by user.
-	monotone --db=$DB heads --branch=$BRANCH \
+	$MTN --db=$DB heads --branch=$BRANCH \
 	  | sed -n -r -e 's/^([^ ]+) ([^ ]+) ([^ ]+)$/\1 \"\2 \3\"/p' \
 	  | xargs dialog --begin 1 2 --menu "Select head" 0 0 0 \
 	  2> $TEMPFILE.input
@@ -534,7 +616,7 @@ do_action_sel()
     while dialog \
 	--backtitle "h:$HEAD b:$BRANCH f:$SHORT_DB" \
 	--menu "Action for $REVISION" 0 60 0 \
-	"L" "Log view of current revision" \
+	"L" "Logview of current revision" \
 	"P" "Diff files from parent" \
 	"W" "Diff files from working copy head" \
 	"S" "Diff files from selected revision" \
@@ -550,14 +632,14 @@ do_action_sel()
 	    # LOG
 	    # 0.19   monotone log --depth=n id file
 	    # 0.19+  monotone log --last=n id file
-	    monotone --db=$DB log $DEPTH_LAST=1 --revision=$REVISION \
+	    $MTN --db=$DB log $DEPTH_LAST=1 --revision=$REVISION \
 	      > $TEMPFILE.change.log || exit 200
 
 	    do_pager $TEMPFILE.change.log
 	    ;;
 	  P)
 	    # DIFF parent
-	    monotone --db=$DB automate parents $REVISION > $TEMPFILE.parents
+	    $MTN --db=$DB automate parents $REVISION > $TEMPFILE.parents
 
 	    if [ `wc -l < $TEMPFILE.parents` -ne 1 ]
 	    then
@@ -586,7 +668,7 @@ do_action_sel()
 	    then
 		dialog --msgbox "No parent found\n$REVISION" 6 45
 	    else
-		monotone --db=$DB diff \
+		$MTN --db=$DB diff \
 		  --revision=$PARENT --revision=$REVISION \
 		  > $TEMPFILE.parent.diff || exit 200
 		do_pager $TEMPFILE.parent.diff
@@ -594,21 +676,21 @@ do_action_sel()
 	    ;;
 	  W)
 	    # DIFF
-	    # monotone diff --revision=id
+	    # $MTN diff --revision=id
 	    if [ "$HEAD" = "$REVISION" ]
 	    then
 		dialog --msgbox "Can't diff with head self\n$HEAD" 6 45
 	    else
 		# exist working copy?
-		if [ -f MT/options ]
+		if [ -f $BK_DIR/options ]
 		then
-		    monotone --db=$DB diff \
+		    $MTN --db=$DB diff \
 		      --revision=$REVISION \
 		      > $TEMPFILE.cwd.diff || exit 200
 		else
 		    # w/o MT dir don't work:
 		    # Help MT with HEAD info ;-)
-		    monotone --db=$DB diff \
+		    $MTN --db=$DB diff \
 		      --revision=$HEAD --revision=$REVISION \
 		      > $TEMPFILE.cwd.diff || exit 200
 		fi
@@ -625,8 +707,8 @@ do_action_sel()
 	    then
 		REV2=`cat $TEMPFILE.revision-select`
 
-		# monotone diff --revision=id1 --revision=id2
-		monotone --db=$DB diff \
+		# $MTN diff --revision=id1 --revision=id2
+		$MTN --db=$DB diff \
 		  --revision=$REV2 --revision=$REVISION \
 		  > $TEMPFILE.ref.diff || exit 200
 		do_pager $TEMPFILE.ref.diff
@@ -635,7 +717,7 @@ do_action_sel()
 	    ;;
 	  C)
 	    # List certs
-	    monotone --db=$DB list certs $REVISION > $TEMPFILE.certs.log \
+	    $MTN --db=$DB list certs $REVISION > $TEMPFILE.certs.log \
 	      || exit 200
 	    do_pager $TEMPFILE.certs.log
 	    ;;
@@ -643,7 +725,7 @@ do_action_sel()
 	   # List changed files
 	   # 0.22: monotone cat revision <id>
 	   # 0.23: monotone automate get_revision <id>
-	   monotone --db=$DB automate get_revision $REVISION > $TEMPFILE.rev.changed \
+	   $MTN --db=$DB automate get_revision $REVISION > $TEMPFILE.rev.changed \
 	     || exit 200
 	   do_pager $TEMPFILE.rev.changed
 	   ;;
@@ -677,8 +759,8 @@ do_automate_ancestors_depth()
 		return 0
 	fi
 
-	let depth++
-	monotone --db=$DB automate parents $head |\
+	depth=$(( $depth+1 ))
+	$MTN --db=$DB automate parents $head |\
 	while read rev
 	do
 	    if ! grep -q -l -e "$rev" $TEMPFILE.ancestors
@@ -687,7 +769,7 @@ do_automate_ancestors_depth()
 		do_automate_ancestors_depth $depth $rev || return $?
 	    fi
 	done
-	let depth--
+	depth=$(( $depth-1 ))
 
 	return 0
 }
@@ -713,9 +795,13 @@ do_revision_sel()
 	echo "Reading ancestors ($HEAD)"
 
 	case $ANCESTORS in
+	    B)	# TODO:
+		# Get ancestors from log, brief format
+		do_log_brief_ancestors $TEMPFILE.certs3tmp
+	    ;;
 	    L)
 		# Get ancestors from log
-		monotone log --brief --last=$CERTS_MAX --revision=$HEAD \
+		$MTN log --brief --last=$CERTS_MAX --revision=$HEAD \
 		  --db=$DB | cut -d ' ' -f 1 > $TEMPFILE.ancestors
 		# Check empty output, if fails
 		test -s $TEMPFILE.ancestors || exit 200
@@ -728,16 +814,18 @@ do_revision_sel()
 	    *)
 		# Get all ancestors
 		echo "$HEAD" > $TEMPFILE.ancestors
-		monotone --db=$DB automate ancestors $HEAD \
+		$MTN --db=$DB automate ancestors $HEAD \
 		  >> $TEMPFILE.ancestors || exit 200
 	    ;;
 	esac
 
-	# Must toposort, if enabled by user, or before tailing
-	if [ "$TOPOSORT" = "T" -o "$CERTS_MAX" -gt 0 -a "$ANCESTORS" != 'L' ]
+	if [ "$ANCESTORS" != "B" ]
 	then
+	    # Must toposort, if enabled by user, or before tailing
+	    if [ "$TOPOSORT" = "T" -o "$CERTS_MAX" -gt 0 -a "$ANCESTORS" != "L" ]
+	    then
 		echo "Toposort..."
-		monotone --db=$DB automate toposort `cat $TEMPFILE.ancestors` \
+		$MTN --db=$DB automate toposort `cat $TEMPFILE.ancestors` \
 		  > $TEMPFILE.toposort || exit 200
 
 		if [ "$CERTS_MAX" -gt 0 ]
@@ -747,12 +835,13 @@ do_revision_sel()
 			  > $TEMPFILE.toposort2
 			mv $TEMPFILE.toposort2 $TEMPFILE.toposort
 		fi
-	else
+	    else
 		mv $TEMPFILE.ancestors $TEMPFILE.toposort
-	fi
+	    fi
 
-	# Reading revisions and fill with date, names and changelog
-	fill_date_key $TEMPFILE.toposort $TEMPFILE.certs3tmp
+	    # Reading revisions and fill with date, names and changelog
+	    fill_date_key $TEMPFILE.toposort $TEMPFILE.certs3tmp
+	fi
 
 	case $TOPOSORT in
 	    D)
@@ -809,7 +898,7 @@ do_revision_sel()
 
 	# Error, on "monotone automate parents XXXXXX", if short revision.  :-(
 	# Expand revision here, if short revision (is alway short now)
-	REVISION=`monotone --db=$DB complete revision $SHORT_REV`
+	REVISION=`$MTN --db=$DB complete revision $SHORT_REV`
 
 	# OK Button: Sub Menu
 	do_action_sel
@@ -860,7 +949,7 @@ do_config()
 	  P)
 	    # Setup for PAGER
 	    if dialog --inputbox \
-		"Config for pipe pager\nuse in sample: \"monotone log | less\"" \
+		"Config for pipe pager\nuse in sample: \"$MTN log | less\"" \
 		9 70 "$PAGER" 2> $TEMPFILE.input
 	    then
 		PAGER=`cat $TEMPFILE.input`
@@ -875,7 +964,7 @@ do_config()
 	    if dialog --default-item "$TOPOSORT" \
 		--menu "Sort revisions by" 0 0 0 \
 		"T" "Toposort, oldest top (from Monotone)" \
-		"D" "Date/Time, oldest top" \
+		"D" "Date/Time, oldest top (internal function)" \
 		"R" "Reverse Date/Time (reverse toposort)" \
 		"N" "None. Simple get from list" \
 		2> $TEMPFILE.input
@@ -926,7 +1015,7 @@ do_config()
 	    # Author coloring
 	    if dialog --default-item \
 		"`test -n \"$FORMAT_COLOR\" && echo \"yes\" || echo \"no\"`" \
-		--menu "Color author in selecetion" 0 0 0 \
+		--menu "Color author in selection" 0 0 0 \
 		"yes" "author is color" \
 		"no" "author has no special color" \
 		2> $TEMPFILE.input
@@ -935,7 +1024,7 @@ do_config()
 		then
 		    dialog --colors \
 		     --default-item "$FORMAT_COLOR" \
-		     --menu "Selecet color for author" 0 0 0 \
+		     --menu "Select color for author" 0 0 0 \
 			"A" "Automatic color" \
 			"\\Z0" "\Z0Color\Zn 0" \
 			"\\Z1" "\Z1Color\Zn 1" \
@@ -977,12 +1066,23 @@ do_config()
 	    # How to get ancestors
 	    if dialog --default-item "$ANCESTORS" \
 		--menu "Get ancestors by using" 0 0 0 \
-		"L" "Monotone \"log --brief\" (fastest)" \
+		"B" "Brief log, without changelog (fastest)" \
+		"L" "Monotone \"log --brief\" with changelog" \
 		"I" "Internal function with depth limit (faster)" \
 		"A" "Monotone \"automate ancestor\" (save mode, very slow)" \
 		2> $TEMPFILE.input
 	    then
 		ANCESTORS=`cat $TEMPFILE.input`
+
+		if [ "$ANCESTORS" = "A" ]
+		then
+		    # functions don't work with big database
+		    dialog --title " Info " --msgbox \
+"\"automate ancestor\":
+
+Big database can overflow command line.
+It's saver to use internal function (I)." 0 0
+		fi
 	    fi
 	    ;;
 	  C)
@@ -996,7 +1096,7 @@ do_config()
 	    # Save environment
 	    cat > $CONFIGFILE << EOF
 # File: ~/.mtbrowserc
-# Created for monotone $MT_MAJOR.$MT_MINOR
+# Created for Monotone $MT_MAJOR.$MT_MINOR
 
 DB="$DB"
 BRANCH="$BRANCH"
@@ -1029,7 +1129,7 @@ EOF
 	if [ $ANCESTORS = "L" -a \( $MT_MAJOR -eq 0 -a $MT_MINOR -lt 20 \) ]
 	then
 	    dialog --title " Info " --msgbox \
-"\"log brief\" is only supported on monotone version 0.20 or newer
+"\"log brief\" is only supported on Monotone version 0.20 or newer
 Automatic corrected" 0 0
 	    ANCESTORS="I"
 	fi
@@ -1039,7 +1139,8 @@ Automatic corrected" 0 0
 	    # functions don't work without limit
 	    dialog --title " Info " --msgbox \
 "Certs limit in Select-List:
-negative or zero last not allowed
+
+Negative or zero limit not allowed
 Automatic corrected" 0 0
 	    CERTS_MAX=20
 	fi
@@ -1065,15 +1166,29 @@ then
     exit -1
 fi
 
-# Get monotone version
-MT_MAJOR=`monotone --version | sed -r -e 's/^.+ ([0-9]{1,2})\.([0-9]{1,2}) .+$/\1/'`
-MT_MINOR=`monotone --version | sed -r -e 's/^.+ ([0-9]{1,2})\.([0-9]{1,2}) .+$/\2/'`
+# Save args
+arg1=$1
+
+# Check for binary
+if [ -z "`which $MTN 2>/dev/null`" ]
+then
+    if [ -n "`which monotone 2>/dev/null`" ]
+    then
+	# fall back to older binary name
+	MTN="monotone"
+    fi
+fi
+
+# Get $MTN version
+set -- `$MTN --version | sed -n -r -e 's/^.+ ([0-9]+)\.([0-9]+).+$/\1 \2/p'`
+MT_MAJOR=$1
+MT_MINOR=$2
 
 if [ -z "$MT_MAJOR" -o -z "$MT_MINOR" ]
 then
     # Hm, need this here
     echo
-    echo "monotone - distributed version control system."
+    echo "$MTN - distributed version control system."
     echo "Can't execute!"
     echo
     exit -1
@@ -1089,6 +1204,71 @@ else
     DEPTH_LAST="--last"
 fi
 
+# New bookkeeping directory is _MTN, in 0.26+
+if [ $MT_MAJOR -eq 0 -a $MT_MINOR -lt 26 ]
+then
+    BK_DIR="MT"
+else
+    BK_DIR="_MTN"
+fi
+
+# exist working copy?
+if [ -f $BK_DIR/options ]
+then
+    # Read parameters from file
+    #  branch "mtbrowse"
+    #database "/home/hn/mtbrowse.db"
+    #     key ""
+
+    eval `sed -n -r \
+      -e 's/^[ ]*(branch) \"([^\"]+)\"$/\1=\2/p' \
+      -e 's/^[ ]*(database) \"([^\"]+)\"$/\1=\2/p' < $BK_DIR/options`
+
+    if [ -n "$database" ]
+    then
+	DB=$database
+	BRANCH=$branch
+    fi
+fi
+
+
+# Simple program args supported
+if [ -n "$arg1" ]
+then
+    case $arg1 in
+      --version)
+	echo "mtbrowse $VERSION"
+	exit 0
+      ;;
+      --help|-h)
+	echo "mtbrowse [dbfile]"
+	exit 0
+      ;;
+      *)
+	# Databasefile from command line
+	DB="$arg1"
+	unset BRANCH
+
+	# MT change the options, if you continue with other DB here!
+	if [ -f $BK_DIR/options ]
+	then
+	    if ! dialog --cr-wrap \
+		--title " *********** WARNING! ********** " \
+		--defaultno --colors --yesno "
+Your \Zb\Z1$BK_DIR/options\Zn will be overwrite, if
+continue with different DB file or branch
+in exist working directory!
+
+YES confirm  /  NO abbort" 0 0
+	    then
+		echo "abbort"
+		exit 1
+	    fi
+	fi
+      ;;
+    esac
+fi
+
 mkdir -p $TEMPDIR
 
 while dialog \
@@ -1127,9 +1307,9 @@ do
 
 	    if [ `echo "$REVISION" | wc -L` -lt 40 ]
 	    then
-		# Error, on "monotone automate parents XXXXXX", if short revision.  :-(
+		# Error, on "$MTN automate parents XXXXXX", if short revision.  :-(
 		# Expand revision here, if short revision
-		REVISION=`monotone --db=$DB complete revision $REVISION`
+		REVISION=`$MTN --db=$DB complete revision $REVISION`
 	    fi
 
 	    do_action_sel
@@ -1186,7 +1366,7 @@ do
 	    $DB -nt $TEMPFILE.changelog.$BRANCH ]
 	then
 	    echo "Reading log...($BRANCH)"
-	    monotone --db=$DB log --revision=$HEAD \
+	    $MTN --db=$DB log --revision=$HEAD \
 	      > $TEMPFILE.changelog.$BRANCH || exit 200
 	fi
 	cp $TEMPFILE.changelog.$BRANCH $TEMPFILE.change.log
@@ -1195,19 +1375,19 @@ do
       t)
 	# List Tags
 	echo "Reading Tags..."
-	monotone --db=$DB list tags > $TEMPFILE.tags.log || exit 200
+	$MTN --db=$DB list tags > $TEMPFILE.tags.log || exit 200
 	do_pager $TEMPFILE.tags.log
 	;;
       h)
 	# if not branch known, ask user
 	do_branch_sel check
 
-	monotone --db=$DB heads --branch=$BRANCH > $TEMPFILE.txt || exit 200
+	$MTN --db=$DB heads --branch=$BRANCH > $TEMPFILE.txt || exit 200
 	do_pager $TEMPFILE.txt
 	;;
       k)
 	# List keys
-	monotone --db=$DB list keys > $TEMPFILE.txt || exit 200
+	$MTN --db=$DB list keys > $TEMPFILE.txt || exit 200
 	do_pager $TEMPFILE.txt
 	;;
       X)
