@@ -46,8 +46,6 @@
 // defined in schema.sql, converted to header:
 #include "schema.h"
 
-// defined in views.sql, converted to header:
-#include "views.h"
 // this file defines a public, typed interface to the database.
 // the database class encapsulates all knowledge about sqlite,
 // the schema, and all SQL statements used to access the schema.
@@ -312,7 +310,6 @@ database::sql(bool init, bool migrating_format)
 
       check_schema();
       install_functions(__app);
-      install_views();
 
       if (!migrating_format)
         check_format();
@@ -1597,18 +1594,18 @@ void
 database::get_revision_manifest(revision_id const & rid,
                                manifest_id & mid)
 {
-  revision_set rev;
+  revision_t rev;
   get_revision(rid, rev);
   mid = rev.new_manifest;
 }
 
 void
 database::get_revision(revision_id const & id,
-                       revision_set & rev)
+                       revision_t & rev)
 {
   revision_data d;
   get_revision(id, d);
-  read_revision_set(d, rev);
+  read_revision(d, rev);
 }
 
 void
@@ -1639,7 +1636,7 @@ void
 database::deltify_revision(revision_id const & rid)
 {
   transaction_guard guard(*this);
-  revision_set rev;
+  revision_t rev;
   MM(rev);
   MM(rid);
   get_revision(rid, rev);
@@ -1676,7 +1673,7 @@ database::deltify_revision(revision_id const & rid)
 
 void
 database::put_revision(revision_id const & new_id,
-                       revision_set const & rev)
+                       revision_t const & rev)
 {
   MM(new_id);
   MM(rev);
@@ -1687,7 +1684,7 @@ database::put_revision(revision_id const & new_id,
   rev.check_sane();
   revision_data d;
   MM(d.inner());
-  write_revision_set(rev, d);
+  write_revision(rev, d);
 
   // Phase 1: confirm the revision makes sense
   {
@@ -1740,8 +1737,8 @@ void
 database::put_revision(revision_id const & new_id,
                        revision_data const & dat)
 {
-  revision_set rev;
-  read_revision_set(dat, rev);
+  revision_t rev;
+  read_revision(dat, rev);
   put_revision(new_id, rev);
 }
 
@@ -2023,28 +2020,6 @@ database::install_functions(app_state * app)
                            NULL, NULL) == 0);
 }
 
-void
-database::install_views()
-{
-  // we don't currently use any views. re-enable this code if you find a
-  // compelling reason to use views.
-
-  /*
-  // delete any existing views
-  results res;
-  fetch(res, one_col, any_rows,
-        "SELECT name FROM sqlite_master WHERE type='view'");
-
-  for (size_t i = 0; i < res.size(); ++i)
-    {
-      string drop = "DROP VIEW " + res[i][0];
-      execute(drop.c_str());
-    }
-  // register any views we're going to use
-  execute(views_constant);
-  */
-}
-
 void
 database::get_certs(vector<cert> & certs,
                     string const & table)
