@@ -8,11 +8,13 @@
 // some of its text into this file and am relicensing my derivative work
 // (this file) copyright (C) 2004 graydon hoare, as LGPL also.
 
-#include <config.h>
+#include "config.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <unistd.h>
 
 #include "file_io.hh"
 #include "botan/botan.h"
@@ -21,13 +23,15 @@
 #define O_BINARY 0
 #endif
 
+using std::string;
+
 int
-monotone_mkstemp(std::string &tmpl)
+monotone_mkstemp(string &tmpl)
 {
   unsigned int len = 0;
   int i = 0;
   int count = 0, fd = -1;
-  std::string tmp;
+  string tmp;
 
   static const char letters[] =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
@@ -43,15 +47,27 @@ monotone_mkstemp(std::string &tmpl)
 
       for (i = 0; i < 6; ++i)
         tmp.append(1, letters[Botan::Global_RNG::random(Botan::Nonce) % NLETTERS]);
-      fd = open(tmp.c_str(), O_RDWR | O_CREAT | O_EXCL | O_BINARY, 0600);      
-      if (fd >= 0)
+#ifdef _MSC_VER
+      fd = _open(tmp.c_str(), _O_RDWR | _O_CREAT | _O_EXCL | _O_BINARY, 0600);
+#else
+	  fd = open(tmp.c_str(), O_RDWR | O_CREAT | O_EXCL | O_BINARY, 0600);
+#endif
+	  if (fd >= 0)
       {
         tmpl = tmp;
         return fd;
       }
       else if (errno != EEXIST)
         break;
-    }  
+    }
   return -1;
 }
 
+
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
