@@ -1,13 +1,16 @@
-// copyright (C) 2004 graydon hoare <graydon@pobox.com>
-// all rights reserved.
-// licensed to the public under the terms of the GNU GPL (>= 2)
-// see the file COPYING for details
+// Copyright (C) 2004 Graydon Hoare <graydon@pobox.com>
+//
+// This program is made available under the GNU GPL version 2.0 or
+// greater. See the accompanying file COPYING for details.
+//
+// This program is distributed WITHOUT ANY WARRANTY; without even the
+// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+// PURPOSE.
 
 #include <string>
 #include <vector>
 #include <utility>
 
-#include "adler32.hh"
 #include "constants.hh"
 #include "netcmd.hh"
 #include "netio.hh"
@@ -16,10 +19,10 @@
 #include "transforms.hh"
 #include "hmac.hh"
 
-using namespace std;
+using std::string;
 
-static netcmd_item_type 
-read_netcmd_item_type(string const & in, 
+static netcmd_item_type
+read_netcmd_item_type(string const & in,
                       size_t & pos,
                       string const & name)
 {
@@ -27,36 +30,34 @@ read_netcmd_item_type(string const & in,
   switch (tmp)
     {
     case static_cast<u8>(revision_item):
-      return revision_item;      
-    case static_cast<u8>(manifest_item):
-      return manifest_item;
+      return revision_item;
     case static_cast<u8>(file_item):
       return file_item;
     case static_cast<u8>(cert_item):
       return cert_item;
     case static_cast<u8>(key_item):
-      return key_item;      
+      return key_item;
     case static_cast<u8>(epoch_item):
-      return epoch_item;      
+      return epoch_item;
     default:
-      throw bad_decode(F("unknown item type 0x%x for '%s'") 
+      throw bad_decode(F("unknown item type 0x%x for '%s'")
                        % static_cast<int>(tmp) % name);
     }
 }
 
 netcmd::netcmd() : version(constants::netcmd_current_protocol_version),
-                   cmd_code(bye_cmd)
+                   cmd_code(error_cmd)
 {}
 
-size_t netcmd::encoded_size() 
+size_t netcmd::encoded_size()
 {
   string tmp;
   insert_datum_uleb128<size_t>(payload.size(), tmp);
   return 1 + 1 + tmp.size() + payload.size() + 4;
 }
 
-bool 
-netcmd::operator==(netcmd const & other) const 
+bool
+netcmd::operator==(netcmd const & other) const
 {
   return version == other.version &&
     cmd_code == other.cmd_code &&
@@ -64,7 +65,7 @@ netcmd::operator==(netcmd const & other) const
 }
 
 // note: usher_reply_cmd does not get included in the hmac.
-void 
+void
 netcmd::write(string & out, chained_hmac & hmac) const
 {
   size_t oldlen = out.size();
@@ -72,7 +73,7 @@ netcmd::write(string & out, chained_hmac & hmac) const
   out += static_cast<char>(cmd_code);
   insert_variable_length_string(payload, out);
 
-  if (cmd_code != usher_reply_cmd)
+  if (hmac.is_active() && cmd_code != usher_reply_cmd)
     {
       string digest = hmac.process(out, oldlen);
       I(hmac.hmac_length == constants::netsync_hmac_value_length_in_bytes);
@@ -81,7 +82,7 @@ netcmd::write(string & out, chained_hmac & hmac) const
 }
 
 // note: usher_cmd does not get included in the hmac.
-bool 
+bool
 netcmd::read(string_queue & inbuf, chained_hmac & hmac)
 {
   size_t pos = 0;
@@ -90,86 +91,103 @@ netcmd::read(string_queue & inbuf, chained_hmac & hmac)
     return false;
 
   u8 extracted_ver = extract_datum_lsb<u8>(inbuf, pos, "netcmd protocol number");
-  if (extracted_ver != version)
-    throw bad_decode(F("protocol version mismatch: wanted '%d' got '%d'") 
-                     % widen<u32,u8>(version)
-                     % widen<u32,u8>(extracted_ver));
-  version = extracted_ver;
 
   u8 cmd_byte = extract_datum_lsb<u8>(inbuf, pos, "netcmd code");
   switch (cmd_byte)
     {
     case static_cast<u8>(hello_cmd):
+    case static_cast<u8>(bye_cmd):
     case static_cast<u8>(anonymous_cmd):
     case static_cast<u8>(auth_cmd):
     case static_cast<u8>(error_cmd):
-    case static_cast<u8>(bye_cmd):
     case static_cast<u8>(confirm_cmd):
     case static_cast<u8>(refine_cmd):
     case static_cast<u8>(done_cmd):
-    case static_cast<u8>(send_data_cmd):
-    case static_cast<u8>(send_delta_cmd):
     case static_cast<u8>(data_cmd):
     case static_cast<u8>(delta_cmd):
-    case static_cast<u8>(nonexistant_cmd):
     case static_cast<u8>(usher_cmd):
       cmd_code = static_cast<netcmd_code>(cmd_byte);
       break;
     default:
-      throw bad_decode(F("unknown netcmd code 0x%x") % widen<u32,u8>(cmd_byte));
+      // if the versions don't match, we will throw the more descriptive
+      // error immediately after this switch.
+      if (extracted_ver == version)
+        throw bad_decode(F("unknown netcmd code 0x%x")
+                          % widen<u32,u8>(cmd_byte));
     }
+  // Ignore the version on usher_cmd packets.
+  if (extracted_ver != version && cmd_code != usher_cmd)
+    throw bad_decode(F("protocol version mismatch: wanted '%d' got '%d'\n"
+                       "%s")
+                     % widen<u32,u8>(version)
+                     % widen<u32,u8>(extracted_ver)
+                     % ((version < extracted_ver)
+                        ? _("the remote side has a newer, incompatible version of monotone")
+                        : _("the remote side has an older, incompatible version of monotone")));
 
   // check to see if we have even enough bytes for a complete uleb128
   size_t payload_len = 0;
   if (!try_extract_datum_uleb128<size_t>(inbuf, pos, "netcmd payload length",
       payload_len))
       return false;
-  
+
   // they might have given us a bogus size
   if (payload_len > constants::netcmd_payload_limit)
     throw bad_decode(F("oversized payload of '%d' bytes") % payload_len);
-  
+
   // there might not be enough data yet in the input buffer
   unsigned int minsize;
-  if (cmd_code == usher_cmd)
-    minsize = pos + payload_len;
-  else
+  if (hmac.is_active() && cmd_code != usher_cmd)
     minsize = pos + payload_len + constants::netsync_hmac_value_length_in_bytes;
+  else
+    minsize = pos + payload_len;
+
   if (inbuf.size() < minsize)
     {
       return false;
     }
 
-  // grab it before the data gets munged
-  I(hmac.hmac_length == constants::netsync_hmac_value_length_in_bytes);
   string digest;
-  if (cmd_code != usher_cmd)
-    digest = hmac.process(inbuf, 0, pos + payload_len);
+  string cmd_digest;
+
+  if (hmac.is_active() && cmd_code != usher_cmd)
+    {
+      // grab it before the data gets munged
+      I(hmac.hmac_length == constants::netsync_hmac_value_length_in_bytes);	
+      digest = hmac.process(inbuf, 0, pos + payload_len);
+    }
 
   payload = extract_substring(inbuf, pos, payload_len, "netcmd payload");
 
-  // they might have given us bogus data
-  string cmd_digest;
-  if (cmd_code != usher_cmd)
-    cmd_digest = extract_substring(inbuf, pos, 
-        constants::netsync_hmac_value_length_in_bytes,
-                                          "netcmd HMAC");
-  inbuf.pop_front(pos);
-  if (cmd_digest != digest)
-    throw bad_decode(F("bad HMAC checksum (got %s, wanted %s)\n"
-                       "this suggests data was corrupted in transit\n")
-                     % encode_hexenc(cmd_digest)
-                     % encode_hexenc(digest));
+  if (hmac.is_active() && cmd_code != usher_cmd)
+    {
+      // they might have given us bogus data
+      cmd_digest = extract_substring(inbuf, pos,
+				     constants::netsync_hmac_value_length_in_bytes,
+				     "netcmd HMAC");
+    }
 
-  return true;    
+  inbuf.pop_front(pos);
+
+  if (hmac.is_active()
+      && cmd_code != usher_cmd
+      && cmd_digest != digest)
+    {
+      throw bad_decode(F("bad HMAC checksum (got %s, wanted %s)\n"
+			 "this suggests data was corrupted in transit\n")
+		       % encode_hexenc(cmd_digest)
+		       % encode_hexenc(digest));
+    }
+
+  return true;
 }
 
 ////////////////////////////////////////////
 // payload reader/writer functions follow //
 ////////////////////////////////////////////
 
-void 
-netcmd::read_error_cmd(std::string & errmsg) const
+void
+netcmd::read_error_cmd(string & errmsg) const
 {
   size_t pos = 0;
   // syntax is: <errmsg:vstr>
@@ -177,8 +195,8 @@ netcmd::read_error_cmd(std::string & errmsg) const
   assert_end_of_buffer(payload, pos, "error netcmd payload");
 }
 
-void 
-netcmd::write_error_cmd(std::string const & errmsg)
+void
+netcmd::write_error_cmd(string const & errmsg)
 {
   cmd_code = error_cmd;
   payload.clear();
@@ -186,7 +204,7 @@ netcmd::write_error_cmd(std::string const & errmsg)
 }
 
 
-void 
+void
 netcmd::read_hello_cmd(rsa_keypair_id & server_keyname,
                        rsa_pub_key & server_key,
                        id & nonce) const
@@ -201,12 +219,12 @@ netcmd::read_hello_cmd(rsa_keypair_id & server_keyname,
                                  "hello netcmd, server key");
   server_key = rsa_pub_key(sk_str);
   nonce = id(extract_substring(payload, pos,
-                               constants::merkle_hash_length_in_bytes, 
+                               constants::merkle_hash_length_in_bytes,
                                "hello netcmd, nonce"));
   assert_end_of_buffer(payload, pos, "hello netcmd payload");
 }
 
-void 
+void
 netcmd::write_hello_cmd(rsa_keypair_id const & server_keyname,
                         rsa_pub_key const & server_key,
                         id const & nonce)
@@ -220,6 +238,25 @@ netcmd::write_hello_cmd(rsa_keypair_id const & server_keyname,
 }
 
 
+void
+netcmd::read_bye_cmd(u8 & phase) const
+{
+  size_t pos = 0;
+  // syntax is: <phase: 1 byte>
+  phase = extract_datum_lsb<u8>(payload, pos, "bye netcmd, phase number");
+  assert_end_of_buffer(payload, pos, "bye netcmd payload");
+}
+
+
+void
+netcmd::write_bye_cmd(u8 phase)
+{
+  cmd_code = bye_cmd;
+  payload.clear();
+  payload += phase;
+}
+
+
 void
 netcmd::read_anonymous_cmd(protocol_role & role,
                            utf8 & include_pattern,
@@ -234,7 +271,7 @@ netcmd::read_anonymous_cmd(protocol_role & role,
       && role_byte != static_cast<u8>(source_and_sink_role))
     throw bad_decode(F("unknown role specifier %d") % widen<u32,u8>(role_byte));
   role = static_cast<protocol_role>(role_byte);
-  std::string pattern_string;
+  string pattern_string;
   extract_variable_length_string(payload, pattern_string, pos,
                                  "anonymous(hmac) netcmd, include_pattern");
   include_pattern = utf8(pattern_string);
@@ -255,18 +292,18 @@ netcmd::write_anonymous_cmd(protocol_role role,
                             rsa_oaep_sha_data const & hmac_key_encrypted)
 {
   cmd_code = anonymous_cmd;
-  payload = static_cast<char>(role);
+  payload += static_cast<char>(role);
   insert_variable_length_string(include_pattern(), payload);
   insert_variable_length_string(exclude_pattern(), payload);
   insert_variable_length_string(hmac_key_encrypted(), payload);
 }
 
-void 
-netcmd::read_auth_cmd(protocol_role & role, 
+void
+netcmd::read_auth_cmd(protocol_role & role,
                       utf8 & include_pattern,
                       utf8 & exclude_pattern,
-                      id & client, 
-                      id & nonce1, 
+                      id & client,
+                      id & nonce1,
                       rsa_oaep_sha_data & hmac_key_encrypted,
                       string & signature) const
 {
@@ -280,7 +317,7 @@ netcmd::read_auth_cmd(protocol_role & role,
       && role_byte != static_cast<u8>(source_and_sink_role))
     throw bad_decode(F("unknown role specifier %d") % widen<u32,u8>(role_byte));
   role = static_cast<protocol_role>(role_byte);
-  std::string pattern_string;
+  string pattern_string;
   extract_variable_length_string(payload, pattern_string, pos,
                                  "auth(hmac) netcmd, include_pattern");
   include_pattern = utf8(pattern_string);
@@ -288,10 +325,10 @@ netcmd::read_auth_cmd(protocol_role & role,
                                  "auth(hmac) netcmd, exclude_pattern");
   exclude_pattern = utf8(pattern_string);
   client = id(extract_substring(payload, pos,
-                                constants::merkle_hash_length_in_bytes, 
+                                constants::merkle_hash_length_in_bytes,
                                 "auth(hmac) netcmd, client identifier"));
   nonce1 = id(extract_substring(payload, pos,
-                                constants::merkle_hash_length_in_bytes, 
+                                constants::merkle_hash_length_in_bytes,
                                 "auth(hmac) netcmd, nonce1"));
   string hmac_key;
   extract_variable_length_string(payload, hmac_key, pos,
@@ -314,7 +351,7 @@ netcmd::write_auth_cmd(protocol_role role,
   cmd_code = auth_cmd;
   I(client().size() == constants::merkle_hash_length_in_bytes);
   I(nonce1().size() == constants::merkle_hash_length_in_bytes);
-  payload = static_cast<char>(role);
+  payload += static_cast<char>(role);
   insert_variable_length_string(include_pattern(), payload);
   insert_variable_length_string(exclude_pattern(), payload);
   payload += client();
@@ -329,7 +366,7 @@ netcmd::read_confirm_cmd() const
   size_t pos = 0;
   assert_end_of_buffer(payload, pos, "confirm netcmd payload");
 }
-  
+
 void
 netcmd::write_confirm_cmd()
 {
@@ -337,104 +374,60 @@ netcmd::write_confirm_cmd()
   payload.clear();
 }
 
-void 
-netcmd::read_refine_cmd(merkle_node & node) const
+void
+netcmd::read_refine_cmd(refinement_type & ty, merkle_node & node) const
 {
-  // syntax is: <node: a merkle tree node>
-  read_node(payload, node);
+  // syntax is: <u8: refinement type> <node: a merkle tree node>
+  size_t pos = 0;
+  ty = static_cast<refinement_type>
+    (extract_datum_lsb<u8>
+     (payload, pos,
+      "refine netcmd, refinement type"));
+  read_node(payload, pos, node);
+  assert_end_of_buffer(payload, pos, "refine cmd");
 }
 
-void 
-netcmd::write_refine_cmd(merkle_node const & node)
+void
+netcmd::write_refine_cmd(refinement_type ty, merkle_node const & node)
 {
   cmd_code = refine_cmd;
   payload.clear();
+  payload += static_cast<char>(ty);
   write_node(node, payload);
 }
 
-void 
-netcmd::read_done_cmd(size_t & level, netcmd_item_type & type)  const
+void
+netcmd::read_done_cmd(netcmd_item_type & type, size_t & n_items)  const
 {
   size_t pos = 0;
-  // syntax is: <level: uleb128> <type: 1 byte>
-  level = extract_datum_uleb128<size_t>(payload, pos,
-                                        "done netcmd, level number");
+  // syntax is: <type: 1 byte> <n_items: uleb128>
   type = read_netcmd_item_type(payload, pos, "done netcmd, item type");
+  n_items = extract_datum_uleb128<size_t>(payload, pos,
+                                          "done netcmd, item-to-send count");
   assert_end_of_buffer(payload, pos, "done netcmd payload");
 }
 
-void 
-netcmd::write_done_cmd(size_t level, 
-                       netcmd_item_type type)
+void
+netcmd::write_done_cmd(netcmd_item_type type,
+                       size_t n_items)
 {
   cmd_code = done_cmd;
   payload.clear();
-  insert_datum_uleb128<size_t>(level, payload);
   payload += static_cast<char>(type);
+  insert_datum_uleb128<size_t>(n_items, payload);
 }
 
-void 
-netcmd::read_send_data_cmd(netcmd_item_type & type, id & item) const
-{
-  size_t pos = 0;
-  // syntax is: <type: 1 byte> <id: 20 bytes sha1> 
-  type = read_netcmd_item_type(payload, pos, "send_data netcmd, item type");
-  item = id(extract_substring(payload, pos,
-                              constants::merkle_hash_length_in_bytes, 
-                              "send_data netcmd, item identifier"));
-  assert_end_of_buffer(payload, pos, "send_data netcmd payload");
-}
-
-void 
-netcmd::write_send_data_cmd(netcmd_item_type type, id const & item)
-{
-  cmd_code = send_data_cmd;
-  I(item().size() == constants::merkle_hash_length_in_bytes);
-  payload = static_cast<char>(type);
-  payload += item();
-}
-
-void 
-netcmd::read_send_delta_cmd(netcmd_item_type & type,
-                            id & base,
-                            id & ident) const
-{
-  size_t pos = 0;
-  // syntax is: <type: 1 byte> <src: 20 bytes sha1> <dst: 20 bytes sha1>
-  type = read_netcmd_item_type(payload, pos, "send_delta netcmd, item type");
-  base = id(extract_substring(payload, pos,
-                              constants::merkle_hash_length_in_bytes, 
-                              "send_delta netcmd, base item identifier"));
-  ident = id(extract_substring(payload, pos,
-                               constants::merkle_hash_length_in_bytes, 
-                              "send_delta netcmd, ident item identifier"));
-  assert_end_of_buffer(payload, pos, "send_delta netcmd payload");
-}
-
-void 
-netcmd::write_send_delta_cmd(netcmd_item_type type,
-                             id const & base,
-                             id const & ident)
-{
-  cmd_code = send_delta_cmd;
-  I(base().size() == constants::merkle_hash_length_in_bytes);
-  I(ident().size() == constants::merkle_hash_length_in_bytes);
-  payload = static_cast<char>(type);
-  payload += base();
-  payload += ident();
-}
-
-void 
+void
 netcmd::read_data_cmd(netcmd_item_type & type,
                       id & item, string & dat) const
 {
   size_t pos = 0;
-  // syntax is: <type: 1 byte> <id: 20 bytes sha1> 
+  // syntax is: <type: 1 byte> <id: 20 bytes sha1>
   //            <compressed_p1: 1 byte> <dat: vstr>
 
   type = read_netcmd_item_type(payload, pos, "data netcmd, item type");
   item = id(extract_substring(payload, pos,
-                              constants::merkle_hash_length_in_bytes, 
+                              constants::merkle_hash_length_in_bytes,
                               "data netcmd, item identifier"));
 
   dat.clear();
@@ -452,14 +445,14 @@ netcmd::read_data_cmd(netcmd_item_type & type,
   assert_end_of_buffer(payload, pos, "data netcmd payload");
 }
 
-void 
+void
 netcmd::write_data_cmd(netcmd_item_type type,
                        id const & item,
                        string const & dat)
 {
   cmd_code = data_cmd;
   I(item().size() == constants::merkle_hash_length_in_bytes);
-  payload = static_cast<char>(type);
+  payload += static_cast<char>(type);
   payload += item();
   if (dat.size() > constants::netcmd_minimum_bytes_to_bother_with_gzip)
     {
@@ -470,25 +463,25 @@ netcmd::write_data_cmd(netcmd_item_type type,
     }
   else
     {
-      payload += static_cast<char>(0); // compressed flag       
+      payload += static_cast<char>(0); // compressed flag
       insert_variable_length_string(dat, payload);
     }
 }
 
 
-void 
+void
 netcmd::read_delta_cmd(netcmd_item_type & type,
                        id & base, id & ident, delta & del) const
 {
   size_t pos = 0;
   // syntax is: <type: 1 byte> <src: 20 bytes sha1> <dst: 20 bytes sha1>
-  //            <compressed_p: 1 byte> <del: vstr>    
+  //            <compressed_p: 1 byte> <del: vstr>
   type = read_netcmd_item_type(payload, pos, "delta netcmd, item type");
   base = id(extract_substring(payload, pos,
-                              constants::merkle_hash_length_in_bytes, 
+                              constants::merkle_hash_length_in_bytes,
                               "delta netcmd, base identifier"));
   ident = id(extract_substring(payload, pos,
-                               constants::merkle_hash_length_in_bytes, 
+                               constants::merkle_hash_length_in_bytes,
                                "delta netcmd, ident identifier"));
   u8 compressed_p = extract_datum_lsb<u8>(payload, pos,
                                           "delta netcmd, compression flag");
@@ -507,15 +500,15 @@ netcmd::read_delta_cmd(netcmd_item_type & type,
   assert_end_of_buffer(payload, pos, "delta netcmd payload");
 }
 
-void 
+void
 netcmd::write_delta_cmd(netcmd_item_type & type,
-                        id const & base, id const & ident, 
+                        id const & base, id const & ident,
                         delta const & del)
 {
   cmd_code = delta_cmd;
   I(base().size() == constants::merkle_hash_length_in_bytes);
   I(ident().size() == constants::merkle_hash_length_in_bytes);
-  payload = static_cast<char>(type);
+  payload += static_cast<char>(type);
   payload += base();
   payload += ident();
 
@@ -530,46 +523,24 @@ netcmd::write_delta_cmd(netcmd_item_type & type,
     }
   else
     {
-      payload += static_cast<char>(0); // compressed flag       
+      payload += static_cast<char>(0); // compressed flag
       tmp = del();
     }
   I(tmp.size() <= constants::netcmd_payload_limit);
   insert_variable_length_string(tmp, payload);
 }
 
-
-void 
-netcmd::read_nonexistant_cmd(netcmd_item_type & type, id & item) const
-{
-  size_t pos = 0;
-  // syntax is: <type: 1 byte> <id: 20 bytes sha1> 
-  type = read_netcmd_item_type(payload, pos, "nonexistant netcmd, item type");
-  item = id(extract_substring(payload, pos,
-                              constants::merkle_hash_length_in_bytes, 
-                              "nonexistant netcmd, item identifier"));
-  assert_end_of_buffer(payload, pos, "nonexistant netcmd payload");
-}
-
-void 
-netcmd::write_nonexistant_cmd(netcmd_item_type type, id const & item)
-{
-  cmd_code = nonexistant_cmd;
-  I(item().size() == constants::merkle_hash_length_in_bytes);
-  payload = static_cast<char>(type);
-  payload += item();
-}
-
-void 
+void
 netcmd::read_usher_cmd(utf8 & greeting) const
 {
   size_t pos = 0;
-  std::string str;
+  string str;
   extract_variable_length_string(payload, str, pos, "error netcmd, message");
   greeting = utf8(str);
   assert_end_of_buffer(payload, pos, "error netcmd payload");
 }
 
-void 
+void
 netcmd::write_usher_reply_cmd(utf8 const & server, utf8 const & pattern)
 {
   cmd_code = usher_reply_cmd;
@@ -592,39 +563,39 @@ test_netcmd_mac()
   string buf;
   netsync_session_key key(constants::netsync_key_initializer);
   {
-    chained_hmac mac(key);
+    chained_hmac mac(key, true);
     // mutates mac
     out_cmd.write(buf, mac);
     BOOST_CHECK_THROW(in_cmd.read_string(buf, mac), bad_decode);
   }
 
   {
-    chained_hmac mac(key);
+    chained_hmac mac(key, true);
     out_cmd.write(buf, mac);
   }
   buf[0] ^= 0xff;
   {
-    chained_hmac mac(key);
+    chained_hmac mac(key, true);
     BOOST_CHECK_THROW(in_cmd.read_string(buf, mac), bad_decode);
   }
 
   {
-    chained_hmac mac(key);
+    chained_hmac mac(key, true);
     out_cmd.write(buf, mac);
   }
   buf[buf.size() - 1] ^= 0xff;
   {
-    chained_hmac mac(key);
+    chained_hmac mac(key, true);
     BOOST_CHECK_THROW(in_cmd.read_string(buf, mac), bad_decode);
   }
 
   {
-    chained_hmac mac(key);
+    chained_hmac mac(key, true);
     out_cmd.write(buf, mac);
   }
   buf += '\0';
   {
-    chained_hmac mac(key);
+    chained_hmac mac(key, true);
     BOOST_CHECK_THROW(in_cmd.read_string(buf, mac), bad_decode);
   }
 }
@@ -634,26 +605,26 @@ do_netcmd_roundtrip(netcmd const & out_cmd, netcmd & in_cmd, string & buf)
 {
   netsync_session_key key(constants::netsync_key_initializer);
   {
-    chained_hmac mac(key);
+    chained_hmac mac(key, true);
     out_cmd.write(buf, mac);
   }
   {
-    chained_hmac mac(key);
+    chained_hmac mac(key, true);
     BOOST_CHECK(in_cmd.read_string(buf, mac));
   }
   BOOST_CHECK(in_cmd == out_cmd);
 }
 
-void 
+void
 test_netcmd_functions()
 {
-  
-  try 
+
+  try
     {
 
       // error_cmd
       {
-        L(boost::format("checking i/o round trip on error_cmd\n")); 
+        L(FL("checking i/o round trip on error_cmd"));
         netcmd out_cmd, in_cmd;
         string out_errmsg("your shoelaces are untied"), in_errmsg;
         string buf;
@@ -661,21 +632,12 @@ test_netcmd_functions()
         do_netcmd_roundtrip(out_cmd, in_cmd, buf);
         in_cmd.read_error_cmd(in_errmsg);
         BOOST_CHECK(in_errmsg == out_errmsg);
-        L(boost::format("errmsg_cmd test done, buffer was %d bytes\n") % buf.size());
+        L(FL("errmsg_cmd test done, buffer was %d bytes") % buf.size());
       }
 
-      // bye_cmd
-      {
-        L(boost::format("checking i/o round trip on bye_cmd\n"));   
-        netcmd out_cmd, in_cmd;
-        string buf;
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        L(boost::format("bye_cmd test done, buffer was %d bytes\n") % buf.size());
-      }
-      
       // hello_cmd
       {
-        L(boost::format("checking i/o round trip on hello_cmd\n"));
+        L(FL("checking i/o round trip on hello_cmd"));
         netcmd out_cmd, in_cmd;
         string buf;
         rsa_keypair_id out_server_keyname("server@there"), in_server_keyname;
@@ -687,12 +649,26 @@ test_netcmd_functions()
         BOOST_CHECK(in_server_keyname == out_server_keyname);
         BOOST_CHECK(in_server_key == out_server_key);
         BOOST_CHECK(in_nonce == out_nonce);
-        L(boost::format("hello_cmd test done, buffer was %d bytes\n") % buf.size());
+        L(FL("hello_cmd test done, buffer was %d bytes") % buf.size());
+      }
+
+      // bye_cmd
+      {
+        L(FL("checking i/o round trip on bye_cmd"));
+        netcmd out_cmd, in_cmd;
+        u8 out_phase(1), in_phase;
+        string buf;
+
+        out_cmd.write_bye_cmd(out_phase);
+        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
+        in_cmd.read_bye_cmd(in_phase);
+        BOOST_CHECK(in_phase == out_phase);
+        L(FL("bye_cmd test done, buffer was %d bytes") % buf.size());
       }
 
       // anonymous_cmd
       {
-        L(boost::format("checking i/o round trip on anonymous_cmd\n"));
+        L(FL("checking i/o round trip on anonymous_cmd"));
         netcmd out_cmd, in_cmd;
         protocol_role out_role = source_and_sink_role, in_role;
         string buf;
@@ -709,16 +685,16 @@ test_netcmd_functions()
         BOOST_CHECK(in_include_pattern == out_include_pattern);
         BOOST_CHECK(in_exclude_pattern == out_exclude_pattern);
         BOOST_CHECK(in_role == out_role);
-        L(boost::format("anonymous_cmd test done, buffer was %d bytes\n") % buf.size());
+        L(FL("anonymous_cmd test done, buffer was %d bytes") % buf.size());
       }
 
       // auth_cmd
       {
-        L(boost::format("checking i/o round trip on auth_cmd\n"));
+        L(FL("checking i/o round trip on auth_cmd"));
         netcmd out_cmd, in_cmd;
         protocol_role out_role = source_and_sink_role, in_role;
         string buf;
-        id out_client(raw_sha1("happy client day")), out_nonce1(raw_sha1("nonce me amadeus")), 
+        id out_client(raw_sha1("happy client day")), out_nonce1(raw_sha1("nonce me amadeus")),
           in_client, in_nonce1;
         // total cheat, since we don't actually verify that rsa_oaep_sha_data
         // is sensible anywhere here...
@@ -739,25 +715,26 @@ test_netcmd_functions()
         BOOST_CHECK(in_role == out_role);
         BOOST_CHECK(in_include_pattern == out_include_pattern);
         BOOST_CHECK(in_exclude_pattern == out_exclude_pattern);
-        L(boost::format("auth_cmd test done, buffer was %d bytes\n") % buf.size());
+        L(FL("auth_cmd test done, buffer was %d bytes") % buf.size());
       }
 
       // confirm_cmd
       {
-        L(boost::format("checking i/o round trip on confirm_cmd\n"));
+        L(FL("checking i/o round trip on confirm_cmd"));
         netcmd out_cmd, in_cmd;
         string buf;
         out_cmd.write_confirm_cmd();
         do_netcmd_roundtrip(out_cmd, in_cmd, buf);
         in_cmd.read_confirm_cmd();
-        L(boost::format("confirm_cmd test done, buffer was %d bytes\n") % buf.size());
+        L(FL("confirm_cmd test done, buffer was %d bytes") % buf.size());
       }
 
       // refine_cmd
       {
-        L(boost::format("checking i/o round trip on refine_cmd\n"));
+        L(FL("checking i/o round trip on refine_cmd"));
         netcmd out_cmd, in_cmd;
         string buf;
+        refinement_type out_ty (refinement_query), in_ty(refinement_response);
         merkle_node out_node, in_node;
 
         out_node.set_raw_slot(0, id(raw_sha1("The police pulled Kris Kringle over")));
@@ -765,70 +742,37 @@ test_netcmd_functions()
         out_node.set_raw_slot(8, id(raw_sha1("He was arrested for auto theft")));
         out_node.set_raw_slot(15, id(raw_sha1("He was whisked away to jail")));
         out_node.set_slot_state(0, subtree_state);
-        out_node.set_slot_state(3, live_leaf_state);
-        out_node.set_slot_state(8, dead_leaf_state);
+        out_node.set_slot_state(3, leaf_state);
+        out_node.set_slot_state(8, leaf_state);
         out_node.set_slot_state(15, subtree_state);
 
-        out_cmd.write_refine_cmd(out_node);
+        out_cmd.write_refine_cmd(out_ty, out_node);
         do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_refine_cmd(in_node);
+        in_cmd.read_refine_cmd(in_ty, in_node);
+        BOOST_CHECK(in_ty == out_ty);
         BOOST_CHECK(in_node == out_node);
-        L(boost::format("refine_cmd test done, buffer was %d bytes\n") % buf.size());
+        L(FL("refine_cmd test done, buffer was %d bytes") % buf.size());
       }
 
       // done_cmd
       {
-        L(boost::format("checking i/o round trip on done_cmd\n"));
+        L(FL("checking i/o round trip on done_cmd"));
         netcmd out_cmd, in_cmd;
-        size_t out_level(12), in_level;
-        netcmd_item_type out_type(key_item), in_type(manifest_item);
+        size_t out_n_items(12), in_n_items(0);
+        netcmd_item_type out_type(key_item), in_type(revision_item);
         string buf;
 
-        out_cmd.write_done_cmd(out_level, out_type);
+        out_cmd.write_done_cmd(out_type, out_n_items);
         do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_done_cmd(in_level, in_type);
-        BOOST_CHECK(in_level == out_level);
+        in_cmd.read_done_cmd(in_type, in_n_items);
+        BOOST_CHECK(in_n_items == out_n_items);
         BOOST_CHECK(in_type == out_type);
-        L(boost::format("done_cmd test done, buffer was %d bytes\n") % buf.size()); 
-      }
-
-      // send_data_cmd
-      {
-        L(boost::format("checking i/o round trip on send_data_cmd\n"));
-        netcmd out_cmd, in_cmd;
-        netcmd_item_type out_type(file_item), in_type(key_item);
-        id out_id(raw_sha1("avocado is the yummiest")), in_id;
-        string buf;
-
-        out_cmd.write_send_data_cmd(out_type, out_id);
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_send_data_cmd(in_type, in_id);
-        BOOST_CHECK(in_type == out_type);
-        BOOST_CHECK(in_id == out_id);
-        L(boost::format("send_data_cmd test done, buffer was %d bytes\n") % buf.size());
-      }
-
-      // send_delta_cmd
-      {
-        L(boost::format("checking i/o round trip on send_delta_cmd\n"));
-        netcmd out_cmd, in_cmd;
-        netcmd_item_type out_type(file_item), in_type(key_item);
-        id out_head(raw_sha1("when you board an airplane")), in_head;
-        id out_base(raw_sha1("always check the exit locations")), in_base;
-        string buf;
-
-        out_cmd.write_send_delta_cmd(out_type, out_head, out_base);
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_send_delta_cmd(in_type, in_head, in_base);
-        BOOST_CHECK(in_type == out_type);
-        BOOST_CHECK(in_head == out_head);
-        BOOST_CHECK(in_base == out_base);
-        L(boost::format("send_delta_cmd test done, buffer was %d bytes\n") % buf.size());
+        L(FL("done_cmd test done, buffer was %d bytes") % buf.size());
       }
 
       // data_cmd
       {
-        L(boost::format("checking i/o round trip on data_cmd\n"));
+        L(FL("checking i/o round trip on data_cmd"));
         netcmd out_cmd, in_cmd;
         netcmd_item_type out_type(file_item), in_type(key_item);
         id out_id(raw_sha1("tuna is not yummy")), in_id;
@@ -839,12 +783,12 @@ test_netcmd_functions()
         in_cmd.read_data_cmd(in_type, in_id, in_dat);
         BOOST_CHECK(in_id == out_id);
         BOOST_CHECK(in_dat == out_dat);
-        L(boost::format("data_cmd test done, buffer was %d bytes\n") % buf.size());
+        L(FL("data_cmd test done, buffer was %d bytes") % buf.size());
       }
 
       // delta_cmd
       {
-        L(boost::format("checking i/o round trip on delta_cmd\n"));
+        L(FL("checking i/o round trip on delta_cmd"));
         netcmd out_cmd, in_cmd;
         netcmd_item_type out_type(file_item), in_type(key_item);
         id out_head(raw_sha1("your seat cusion can be reused")), in_head;
@@ -859,34 +803,18 @@ test_netcmd_functions()
         BOOST_CHECK(in_head == out_head);
         BOOST_CHECK(in_base == out_base);
         BOOST_CHECK(in_delta == out_delta);
-        L(boost::format("delta_cmd test done, buffer was %d bytes\n") % buf.size());
-      }
-
-      // nonexistant_cmd
-      {
-        L(boost::format("checking i/o round trip on nonexistant_cmd\n"));
-        netcmd out_cmd, in_cmd;
-        netcmd_item_type out_type(file_item), in_type(key_item);
-        id out_id(raw_sha1("avocado is the yummiest")), in_id;
-        string buf;
-
-        out_cmd.write_nonexistant_cmd(out_type, out_id);
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_nonexistant_cmd(in_type, in_id);
-        BOOST_CHECK(in_type == out_type);
-        BOOST_CHECK(in_id == out_id);
-        L(boost::format("nonexistant_cmd test done, buffer was %d bytes\n") % buf.size());
+        L(FL("delta_cmd test done, buffer was %d bytes") % buf.size());
       }
 
     }
   catch (bad_decode & d)
     {
-      L(boost::format("bad decode exception: '%s'\n") % d.what);
+      L(FL("bad decode exception: '%s'") % d.what);
       throw;
     }
 }
 
-void 
+void
 add_netcmd_tests(test_suite * suite)
 {
   suite->add(BOOST_TEST_CASE(&test_netcmd_functions));
@@ -894,3 +822,11 @@ add_netcmd_tests(test_suite * suite)
 }
 
 #endif // BUILD_UNIT_TESTS
+
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
