@@ -138,25 +138,15 @@ bad_component(string const & component)
 static inline bool
 has_bad_chars(string const & path)
 {
-  static bool bad_chars_init(false);
-  static u8 bad_table[128] = {0};
-  if (UNLIKELY(!bad_chars_init))
-    {
-      string bad_chars = string("\\") + constants::illegal_path_bytes + string(1, '\0');
-      for (string::const_iterator b = bad_chars.begin(); b != bad_chars.end(); b++)
-        {
-          u8 x = (u8)*b;
-          I((x) < sizeof(bad_table));
-          bad_table[x] = 1;
-        }
-      bad_chars_init = true;
-    }
-
-  for (string::const_iterator c = path.begin(); c != path.end(); c++)
+  for (string::const_iterator c = path.begin(); LIKELY(c != path.end()); c++)
     {
+      // char is often a signed type; convert to unsigned to ensure that
+      // bytes 0x80-0xff are considered > 0x1f.
       u8 x = (u8)*c;
-      if (x < sizeof(bad_table) && bad_table[x])
-          return true;
+      // 0x5c is '\\'; we use the hex constant to make the dependency on
+      // ASCII encoding explicit.
+      if (UNLIKELY(x <= 0x1f || x == 0x5c || x == 0x7f))
+        return true;
     }
   return false;
 }
@@ -563,6 +553,13 @@ system_path::system_path(utf8 const & path)
 // utility
 ///////////////////////////////////////////////////////////////////////////
 
+bool
+workspace_root(split_path const & sp)
+{
+  I(null_name(idx(sp,0)));
+  return sp.size() == 1;
+}
+
 void
 dirname_basename(split_path const & sp,
                  split_path & dirname, path_component & basename)
