@@ -182,6 +182,45 @@ function binary_file(name)
    return guess_binary_file_contents(name)
 end
 
+-- given a file name, return a regular expression which will match
+-- lines that name top-level constructs in that file, or "", to disable
+-- matching.
+function get_encloser_pattern(name)
+   -- texinfo has special sectioning commands
+   if (string.find(name, "%.texi$")) then
+      -- sectioning commands in texinfo: @node, @chapter, @top, 
+      -- @((sub)?sub)?section, @unnumbered(((sub)?sub)?sec)?,
+      -- @appendix(((sub)?sub)?sec)?, @(|major|chap|sub(sub)?)heading
+      return ("^@("
+              .. "node|chapter|top"
+              .. "|((sub)?sub)?section"
+              .. "|(unnumbered|appendix)(((sub)?sub)?sec)?"
+              .. "|(major|chap|sub(sub)?)?heading"
+              .. ")")
+   end
+   -- LaTeX has special sectioning commands.  This rule is applied to ordinary
+   -- .tex files too, since there's no reliable way to distinguish those from
+   -- latex files anyway, and there's no good pattern we could use for
+   -- arbitrary plain TeX anyway.
+   if (string.find(name, "%.tex$")
+       or string.find(name, "%.ltx$")
+       or string.find(name, "%.latex$")) then
+      return ("\\\\("
+              .. "part|chapter|paragraph|subparagraph"
+              .. "|((sub)?sub)?section"
+              .. ")")
+   end
+   -- There's no good way to find section headings in raw text, and trying
+   -- just gives distracting output, so don't even try.
+   if (string.find(name, "%.txt$")
+       or string.upper(name) == "README") then
+      return ""
+   end
+   -- This default is correct surprisingly often -- in pretty much any text
+   -- written with code-like indentation.
+   return "^[[:alnum:]$_]"
+end
+
 function edit_comment(basetext, user_log_message)
    local exe = nil
    if (program_exists_in_path("vi")) then exe = "vi" end
@@ -292,7 +331,7 @@ function merge3_meld_cmd(lfile, afile, rfile)
       local path = "meld"
       local ret = execute(path, lfile, afile, rfile)
       if (ret ~= 0) then
-	 io.write(string.format(gettext("Error running merger '%s'\n"), path))
+         io.write(string.format(gettext("Error running merger '%s'\n"), path))
       end
       return ret
    end
@@ -303,12 +342,12 @@ function merge3_tortoise_cmd(lfile, afile, rfile, outfile)
    function()
       local path = "tortoisemerge"
       local ret = execute(path,
-			  string.format("/base:%s", afile),
-			  string.format("/theirs:%s", lfile),
-			  string.format("/mine:%s", rfile),
-			  string.format("/merged:%s", outfile))
+                          string.format("/base:%s", afile),
+                          string.format("/theirs:%s", lfile),
+                          string.format("/mine:%s", rfile),
+                          string.format("/merged:%s", outfile))
       if (ret ~= 0) then
-	 io.write(string.format(gettext("Error running merger '%s'\n"), path))
+         io.write(string.format(gettext("Error running merger '%s'\n"), path))
       end
       return ret
    end
@@ -318,9 +357,9 @@ function merge3_vim_cmd(vim, afile, lfile, rfile, outfile)
    return
    function()
       local ret = execute(vim, "-f", "-d", "-c", string.format("file %s", outfile),
-			  afile, lfile, rfile)
+                          afile, lfile, rfile)
       if (ret ~= 0) then
-	 io.write(string.format(gettext("Error running merger '%s'\n"), vim))
+         io.write(string.format(gettext("Error running merger '%s'\n"), vim))
       end
       return ret
    end
@@ -338,9 +377,9 @@ function merge3_rcsmerge_vim_cmd(merge, vim, lfile, afile, rfile, outfile)
          return 0
       end
       local ret = execute(vim, "-f", "-c", string.format("file %s", outfile),
-			  lfile)
+                          lfile)
       if (ret ~= 0) then
-	 io.write(string.format(gettext("Error running merger '%s'\n"), vim))
+         io.write(string.format(gettext("Error running merger '%s'\n"), vim))
       end
       return ret
    end
@@ -351,9 +390,9 @@ function merge3_emacs_cmd(emacs, lfile, afile, rfile, outfile)
    return 
    function()
       local ret = execute(emacs, "--eval", 
-			  string.format(elisp, lfile, rfile, afile, outfile))
+                          string.format(elisp, lfile, rfile, afile, outfile))
       if (ret ~= 0) then
-	 io.write(string.format(gettext("Error running merger '%s'\n"), emacs))
+         io.write(string.format(gettext("Error running merger '%s'\n"), emacs))
       end
       return ret
    end
@@ -365,15 +404,15 @@ function merge3_xxdiff_cmd(left_path, anc_path, right_path, merged_path,
    function()
       local path = "xxdiff"
       local ret = execute(path, 
-			"--title1", left_path,
-			"--title2", right_path,
-			"--title3", merged_path,
-			lfile, afile, rfile, 
-			"--merge", 
-			"--merged-filename", outfile,
-			"--exit-with-merge-status")
+                        "--title1", left_path,
+                        "--title2", right_path,
+                        "--title3", merged_path,
+                        lfile, afile, rfile, 
+                        "--merge", 
+                        "--merged-filename", outfile,
+                        "--exit-with-merge-status")
       if (ret ~= 0) then
-	 io.write(string.format(gettext("Error running merger '%s'\n"), path))
+         io.write(string.format(gettext("Error running merger '%s'\n"), path))
       end
       return ret
    end
@@ -385,14 +424,14 @@ function merge3_kdiff3_cmd(left_path, anc_path, right_path, merged_path,
    function()
       local path = "kdiff3"
       local ret = execute(path, 
-			  "--L1", anc_path,
-			  "--L2", left_path,
-			  "--L3", right_path,
-			  afile, lfile, rfile, 
-			  "--merge", 
-			  "--o", outfile)
+                          "--L1", anc_path,
+                          "--L2", left_path,
+                          "--L3", right_path,
+                          afile, lfile, rfile, 
+                          "--merge", 
+                          "--o", outfile)
       if (ret ~= 0) then
-	 io.write(string.format(gettext("Error running merger '%s'\n"), path))
+         io.write(string.format(gettext("Error running merger '%s'\n"), path))
       end
       return ret
    end
@@ -404,11 +443,11 @@ function merge3_opendiff_cmd(left_path, anc_path, right_path, merged_path, lfile
       local path = "opendiff"
       -- As opendiff immediately returns, let user confirm manually
       local ret = execute_confirm(path,
-				  lfile,rfile,
-				  "-ancestor",afile,
-				  "-merge",outfile)
+                                  lfile,rfile,
+                                  "-ancestor",afile,
+                                  "-merge",outfile)
       if (ret ~= 0) then
-	 io.write(string.format(gettext("Error running merger '%s'\n"), path))
+         io.write(string.format(gettext("Error running merger '%s'\n"), path))
       end
       return ret
    end
@@ -536,22 +575,22 @@ function merge3 (anc_path, left_path, right_path, merged_path, ancestor, left, r
       if cmd ~=nil 
       then 
          io.write (string.format(gettext("executing external 3-way merge command\n")))
-	 -- cmd() return 0 on success.
+         -- cmd() return 0 on success.
          if cmd () ~= 0
-	 then
-	    ret = nil
-	 else
-	    if tbl.meld_exists 
-	    then 
-	       ret = read_contents_of_file (tbl.afile, "r")
-	    else
-	       ret = read_contents_of_file (tbl.outfile, "r") 
-	    end 
-	    if string.len (ret) == 0 
-	    then 
-	       ret = nil 
-	    end
-	 end
+         then
+            ret = nil
+         else
+            if tbl.meld_exists 
+            then 
+               ret = read_contents_of_file (tbl.afile, "r")
+            else
+               ret = read_contents_of_file (tbl.outfile, "r") 
+            end 
+            if string.len (ret) == 0 
+            then 
+               ret = nil 
+            end
+         end
       else
          io.write (string.format("No external 3-way merge command found.\n"..
             "You may want to check that $EDITOR is set to an editor that supports 3-way merge,\n"..
@@ -706,20 +745,20 @@ function get_netsync_read_permitted(branch, ident)
       if item.name == "pattern" then
          if matches and not cont then return false end
          matches = false
-	 cont = false
+         cont = false
          for j, val in pairs(item.values) do
             if globish_match(val, branch) then matches = true end
          end
       elseif item.name == "allow" then if matches then
          for j, val in pairs(item.values) do
             if val == "*" then return true end
-	    if val == "" and ident == nil then return true end
+            if val == "" and ident == nil then return true end
             if globish_match(val, ident) then return true end
          end
       end elseif item.name == "deny" then if matches then
          for j, val in pairs(item.values) do
             if val == "*" then return false end
-	    if val == "" and ident == nil then return false end
+            if val == "" and ident == nil then return false end
             if globish_match(val, ident) then return false end
          end
       end elseif item.name == "continue" then if matches then
@@ -759,73 +798,77 @@ end
 
 function get_netsync_connect_command(uri, args)
 
-	local argv = nil
-	local quote_patterns = false
+        local argv = nil
+        local quote_patterns = false
 
-	if uri["scheme"] == "ssh" 
-		and uri["host"] 
-		and uri["path"] then
+        if uri["scheme"] == "ssh" 
+                and uri["host"] 
+                and uri["path"] then
 
-		argv = { "ssh" }
-		if uri["user"] then
-			table.insert(argv, "-l")
-			table.insert(argv, uri["user"])
-		end
-		if uri["port"] then
-			table.insert(argv, "-p")
-			table.insert(argv, uri["port"])
-		end
+                argv = { "ssh" }
+                if uri["user"] then
+                        table.insert(argv, "-l")
+                        table.insert(argv, uri["user"])
+                end
+                if uri["port"] then
+                        table.insert(argv, "-p")
+                        table.insert(argv, uri["port"])
+                end
 
-		table.insert(argv, uri["host"])
+                table.insert(argv, uri["host"])
     quote_patterns = true
-	end
-	
-	if uri["scheme"] == "file" and uri["path"] then
-		argv = { }
-	end
+        end
+        
+        if uri["scheme"] == "file" and uri["path"] then
+                argv = { }
+        end
 
-	if argv then
+        if argv then
 
-		table.insert(argv, "mtn")
+                table.insert(argv, get_mtn_command(uri["host"]))
 
-		if args["debug"] then
-			table.insert(argv, "--debug")
-		else
-			table.insert(argv, "--quiet")
-		end
+                if args["debug"] then
+                        table.insert(argv, "--debug")
+                else
+                        table.insert(argv, "--quiet")
+                end
 
-		table.insert(argv, "--db")
-		table.insert(argv, uri["path"])
-		table.insert(argv, "serve")
-		table.insert(argv, "--stdio")
-		table.insert(argv, "--no-transport-auth")
+                table.insert(argv, "--db")
+                table.insert(argv, uri["path"])
+                table.insert(argv, "serve")
+                table.insert(argv, "--stdio")
+                table.insert(argv, "--no-transport-auth")
 
-		-- patterns must be quoted to avoid a remote shell expanding them
-		if args["include"] then
-			local include = args["include"]
-			if quote_patterns then
-				include = "'" .. args["include"] .. "'"
-			end
-			table.insert(argv, include)
-		end
+                -- patterns must be quoted to avoid a remote shell expanding them
+                if args["include"] then
+                        local include = args["include"]
+                        if quote_patterns then
+                                include = "'" .. args["include"] .. "'"
+                        end
+                        table.insert(argv, include)
+                end
 
-		if args["exclude"] then
-			table.insert(argv, "--exclude")
-			local exclude = args["exclude"]
-			if quote_patterns then
-				exclude = "'" .. args["exclude"] .. "'"
-			end
-			table.insert(argv, exclude)
-		end
-	end
-	return argv
+                if args["exclude"] then
+                        table.insert(argv, "--exclude")
+                        local exclude = args["exclude"]
+                        if quote_patterns then
+                                exclude = "'" .. args["exclude"] .. "'"
+                        end
+                        table.insert(argv, exclude)
+                end
+        end
+        return argv
 end
 
 function use_transport_auth(uri)
-	if uri["scheme"] == "ssh" 
-	or uri["scheme"] == "file" then
-		return false
-	else
-		return true
-	end
+        if uri["scheme"] == "ssh" 
+        or uri["scheme"] == "file" then
+                return false
+        else
+                return true
+        end
+end
+
+function get_mtn_command(host)
+        return "mtn"
 end
