@@ -1,65 +1,89 @@
-// -*- mode: C++; c-file-style: "gnu"; indent-tabs-mode: nil -*-
-// copyright (C) 2002, 2003 graydon hoare <graydon@pobox.com>
-// all rights reserved.
-// licensed to the public under the terms of the GNU GPL (>= 2)
-// see the file COPYING for details
+// Copyright (C) 2002 Graydon Hoare <graydon@pobox.com>
+//
+// This program is made available under the GNU GPL version 2.0 or
+// greater. See the accompanying file COPYING for details.
+//
+// This program is distributed WITHOUT ANY WARRANTY; without even the
+// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+// PURPOSE.
 
 // this file contains a couple utilities to deal with the user
 // interface. the global user_interface object 'ui' owns clog, so no
 // writing to it directly!
 
 #include "config.h"
+
 #include "platform.hh"
 #include "sanity.hh"
 #include "ui.hh"
-#include "transforms.hh"
+#include "charset.hh"
+#include "simplestring_xform.hh"
 #include "constants.hh"
 
 #include <iostream>
+#include <fstream>
 #include <iomanip>
+#include <algorithm>
 #include <boost/lexical_cast.hpp>
 
-using namespace std;
+using std::clog;
+using std::cout;
+using std::endl;
+using std::ios_base;
+using std::locale;
+using std::make_pair;
+using std::map;
+using std::max;
+using std::ofstream;
+using std::string;
+using std::vector;
+
 using boost::lexical_cast;
+
 struct user_interface ui;
 
-ticker::ticker(string const & tickname, std::string const & s, size_t mod,
+ticker::ticker(string const & tickname, string const & s, size_t mod,
     bool kilocount) :
   ticks(0),
   mod(mod),
+  total(0),
+  previous_total(0),
   kilocount(kilocount),
-  name(tickname),
-  shortname(s)
+  use_total(false),
+  keyname(tickname),
+  name(_(tickname.c_str())),
+  shortname(s),
+  count_size(0)
 {
-  I(ui.tickers.find(tickname) == ui.tickers.end());
-  ui.tickers.insert(make_pair(tickname,this));
+  I(ui.tickers.find(keyname) == ui.tickers.end());
+  ui.tickers.insert(make_pair(keyname, this));
 }
 
 ticker::~ticker()
 {
-  I(ui.tickers.find(name) != ui.tickers.end());
+  I(ui.tickers.find(keyname) != ui.tickers.end());
   if (ui.some_tick_is_dirty)
     {
       ui.write_ticks();
     }
-  ui.tickers.erase(name);
+  ui.tickers.erase(keyname);
   ui.finish_ticking();
 }
 
-void 
+void
 ticker::operator++()
 {
-  I(ui.tickers.find(name) != ui.tickers.end());
+  I(ui.tickers.find(keyname) != ui.tickers.end());
   ticks++;
   ui.some_tick_is_dirty = true;
   if (ticks % mod == 0)
     ui.write_ticks();
 }
 
-void 
+void
 ticker::operator+=(size_t t)
 {
-  I(ui.tickers.find(name) != ui.tickers.end());
+  I(ui.tickers.find(keyname) != ui.tickers.end());
   size_t old = ticks;
 
   ticks += t;
@@ -80,71 +104,148 @@ tick_write_count::~tick_write_count()
 {
 }
 
-void tick_write_count::write_ticks()
+static string compose_count(ticker *tick, size_t ticks=0)
 {
-  string tickline1, tickline2;
-  bool first_tick = true;
+  string count;
 
-  tickline1 = "monotone: ";
-  tickline2 = "\rmonotone:";
-  
-  unsigned int width;
-  unsigned int minwidth = 7;
-  for (map<string,ticker *>::const_iterator i = ui.tickers.begin();
-       i != ui.tickers.end(); ++i)
+  if (ticks == 0)
     {
-      width = 1 + display_width(utf8(i->second->name));
-      if (!first_tick)
+      ticks = tick->ticks;
+    }
+
+  if (tick->kilocount && ticks)
+    {
+      // automatic unit conversion is enabled
+      float div = 1.0;
+      const char *message;
+
+      if (ticks >= 1073741824)
         {
-          tickline1 += " | ";
-          tickline2 += " |";
+          div = 1073741824;
+          // xgettext: gibibytes (2^30 bytes)
+          message = N_("%.1f G");
         }
-      first_tick = false;
-      if (display_width(utf8(i->second->name)) < minwidth)
+      else if (ticks >= 1048576)
         {
-          tickline1.append(minwidth - display_width(utf8(i->second->name)),' ');
-          width += minwidth - display_width(utf8(i->second->name));
-        }
-      tickline1 += i->second->name;
-      
-      string count;
-      if (i->second->kilocount && i->second->ticks >= 10000)
-        { // automatic unit conversion is enabled
-          float div;
-          const char *message;
-          if (i->second->ticks >= 1048576) {
-          // ticks >=1MB, use Mb
-            div = 1048576;
+          div = 1048576;
           // xgettext: mebibytes (2^20 bytes)
-            message = N_("%.1f M");
-          } else {
-          // ticks <1MB, use kb
-            div = 1024;
+          message = N_("%.1f M");
+        }
+      else if (ticks >= 1024)
+        {
+          div = 1024;
           // xgettext: kibibytes (2^10 bytes)
-            message = N_("%.1f k");
-          }
-          // we reset the mod to the divider, to avoid spurious screen updates
-          i->second->mod = static_cast<int>(div / 10.0);
-          count = (F(message) % (i->second->ticks / div)).str();
+          message = N_("%.1f k");
         }
       else
         {
-          // xgettext: bytes
-          count = (F("%d") % i->second->ticks).str();
+          div = 1;
+          message = "%.0f";
         }
-        
-      if (display_width(utf8(count)) < width)
+      // We reset the mod to the divider, to avoid spurious screen updates.
+      tick->mod = max(static_cast<int>(div / 10.0), 1);
+      count = (F(message) % (ticks / div)).str();
+    }
+  else if (tick->use_total)
+    {
+      count = (F("%d/%d") % ticks % tick->total).str();
+    }
+  else
+    {
+      // xgettext: bytes
+      count = (F("%d") % ticks).str();
+    }
+
+  return count;
+}
+
+void tick_write_count::write_ticks()
+{
+  vector<size_t> tick_widths;
+  vector<string> tick_title_strings;
+  vector<string> tick_count_strings;
+
+  for (map<string,ticker *>::const_iterator i = ui.tickers.begin();
+       i != ui.tickers.end(); ++i)
+    {
+      ticker * tick = i->second;
+
+      if ((tick->count_size == 0 && tick->kilocount)
+          ||
+          (tick->use_total && tick->previous_total != tick->total))
         {
-          tickline2.append(width - display_width(utf8(count)),' ');
+          if (!tick->kilocount && tick->use_total)
+            {
+              // We know that we're going to eventually have 'total'
+              // displayed twice on screen, plus a slash. So we should
+              // pad out this field to that eventual size to avoid
+              // spurious re-issuing of the tick titles as we expand to
+              // the goal.
+              tick->set_count_size(display_width(utf8(compose_count(tick,
+                                                                    tick->total))));
+              tick->previous_total = tick->total;
+            }
+          else
+            {
+              // To find out what the maximum size can be, choose one the
+              // the dividers from compose_count, subtract one and have
+              // compose_count create the count string for that.  Use the
+              // size of the returned count string as an initial size for
+              // this tick.
+              tick->set_count_size(display_width(utf8(compose_count(tick,
+                                                                    1048575))));
+            }
         }
-      else if (display_width(utf8(count)) > width)
+
+      string count(compose_count(tick));
+
+      size_t title_width = display_width(utf8(tick->name));
+      size_t count_width = display_width(utf8(count));
+
+      if (count_width > tick->count_size)
         {
-          // FIXME: not quite right, because substr acts on bytes rather than
-          // characters; but there are always more bytes than characters, so
-          // at worst this will just chop off a little too much.
-          count = count.substr(display_width(utf8(count)) - width);
+          tick->set_count_size(count_width);
         }
-      tickline2 += count;
+
+      size_t max_width = max(title_width, tick->count_size);
+
+      string name;
+      name.append(max_width - title_width, ' ');
+      name.append(tick->name);
+
+      string count2;
+      count2.append(max_width - count_width, ' ');
+      count2.append(count);
+
+      tick_title_strings.push_back(name);
+      tick_count_strings.push_back(count2);
+      tick_widths.push_back(max_width);
+    }
+
+  string tickline1;
+  bool write_tickline1 = !(ui.last_write_was_a_tick
+                           && (tick_widths == last_tick_widths));
+  if (write_tickline1)
+    {
+      // Reissue the titles if the widths have changed.
+      tickline1 = ui.output_prefix();
+      for (size_t i = 0; i < tick_widths.size(); ++i)
+        {
+          if (i != 0)
+            tickline1.append(" | ");
+          tickline1.append(idx(tick_title_strings, i));
+        }
+      last_tick_widths = tick_widths;
+      write_tickline1 = true;
+    }
+
+  // Always reissue the counts.
+  string tickline2 = ui.output_prefix();
+  for (size_t i = 0; i < tick_widths.size(); ++i)
+    {
+      if (i != 0)
+        tickline2.append(" | ");
+      tickline2.append(idx(tick_count_strings, i));
     }
 
   if (!ui.tick_trailer.empty())
@@ -152,15 +253,18 @@ void tick_write_count::write_ticks()
       tickline2 += " ";
       tickline2 += ui.tick_trailer;
     }
-  
+
   size_t curr_sz = display_width(utf8(tickline2));
   if (curr_sz < last_tick_len)
     tickline2.append(last_tick_len - curr_sz, ' ');
   last_tick_len = curr_sz;
 
   unsigned int tw = terminal_width();
-  if(!ui.last_write_was_a_tick)
+  if(write_tickline1)
     {
+      if (ui.last_write_was_a_tick)
+        clog << "\n";
+
       if (tw && display_width(utf8(tickline1)) > tw)
         {
           // FIXME: may chop off more than necessary (because we chop by
@@ -171,13 +275,11 @@ void tick_write_count::write_ticks()
     }
   if (tw && display_width(utf8(tickline2)) > tw)
     {
-      // first character in tickline2 is "\r", which does not take up any
-      // width, so we add 1 to compensate.
       // FIXME: may chop off more than necessary (because we chop by
       // bytes, not by characters)
-      tickline2.resize(tw + 1);
+      tickline2.resize(tw);
     }
-  clog << tickline2;
+  clog << "\r" << tickline2;
   clog.flush();
 }
 
@@ -197,7 +299,7 @@ tick_write_dot::~tick_write_dot()
 
 void tick_write_dot::write_ticks()
 {
-  static const string tickline_prefix = "monotone: ";
+  static const string tickline_prefix = ui.output_prefix();
   string tickline1, tickline2;
   bool first_tick = true;
 
@@ -208,7 +310,7 @@ void tick_write_dot::write_ticks()
     }
   else
     {
-      tickline1 = "monotone: ticks: ";
+      tickline1 = ui.output_prefix() + "ticks: ";
       tickline2 = "\n" + tickline_prefix;
       chars_on_line = tickline_prefix.size();
     }
@@ -263,11 +365,11 @@ user_interface::user_interface() :
   t_writer(0)
 {
   cout.exceptions(ios_base::badbit);
-#ifndef WIN32
+#ifdef SYNC_WITH_STDIO_WORKS
   clog.sync_with_stdio(false);
 #endif
   clog.unsetf(ios_base::unitbuf);
-  if (have_smart_terminal()) 
+  if (have_smart_terminal())
     set_tick_writer(new tick_write_count);
   else
     set_tick_writer(new tick_write_dot);
@@ -278,10 +380,10 @@ user_interface::~user_interface()
   delete t_writer;
 }
 
-void 
+void
 user_interface::finish_ticking()
 {
-  if (tickers.size() == 0 && 
+  if (tickers.size() == 0 &&
       last_write_was_a_tick)
     {
       tick_trailer = "";
@@ -290,13 +392,13 @@ user_interface::finish_ticking()
     }
 }
 
-void 
+void
 user_interface::set_tick_trailer(string const & t)
 {
   tick_trailer = t;
 }
 
-void 
+void
 user_interface::set_tick_writer(tick_writer * t)
 {
   if (t_writer != 0)
@@ -304,7 +406,7 @@ user_interface::set_tick_writer(tick_writer * t)
   t_writer = t;
 }
 
-void 
+void
 user_interface::write_ticks()
 {
   t_writer->write_ticks();
@@ -312,37 +414,56 @@ user_interface::write_ticks()
   some_tick_is_dirty = false;
 }
 
-void 
+void
 user_interface::warn(string const & warning)
 {
   if (issued_warnings.find(warning) == issued_warnings.end())
-    inform("warning: " + warning);
+    {
+      string message;
+      prefix_lines_with(_("warning: "), warning, message);
+      inform(message);
+    }
   issued_warnings.insert(warning);
 }
 
-void 
+void
 user_interface::fatal(string const & fatal)
 {
   inform(F("fatal: %s\n"
            "this is almost certainly a bug in monotone.\n"
-           "please send this error message, the output of 'monotone --full-version',\n"
+           "please send this error message, the output of '%s --full-version',\n"
            "and a description of what you were doing to %s.\n")
-         % fatal % PACKAGE_BUGREPORT);
+         % fatal % prog_name % PACKAGE_BUGREPORT);
 }
 
+void
+user_interface::set_prog_name(string const & name)
+{
+  prog_name = name;
+  I(!prog_name.empty());
+}
 
-static inline string 
+string
+user_interface::output_prefix()
+{
+  if (prog_name.empty()) {
+    return "?: ";
+  }
+  return prog_name + ": ";
+}
+
+static inline string
 sanitize(string const & line)
 {
   // FIXME: you might want to adjust this if you're using a charset
-  // which has safe values in the sub-0x20 range. ASCII, UTF-8, 
+  // which has safe values in the sub-0x20 range. ASCII, UTF-8,
   // and most ISO8859-x sets do not.
   string tmp;
   tmp.reserve(line.size());
   for (size_t i = 0; i < line.size(); ++i)
     {
       if ((line[i] == '\n')
-          || (static_cast<unsigned char>(line[i]) >= static_cast<unsigned char>(0x20) 
+          || (static_cast<unsigned char>(line[i]) >= static_cast<unsigned char>(0x20)
               && line[i] != static_cast<char>(0x7F)))
         tmp += line[i];
       else
@@ -362,11 +483,22 @@ user_interface::ensure_clean_line()
   last_write_was_a_tick = false;
 }
 
-void 
+void
+user_interface::redirect_log_to(system_path const & filename)
+{
+  static ofstream filestr;
+  if (filestr.is_open())
+    filestr.close();
+  filestr.open(filename.as_external().c_str(), ofstream::out | ofstream::app);
+  E(filestr.is_open(), F("failed to open log file '%s'") % filename);
+  clog.rdbuf(filestr.rdbuf());
+}
+
+void
 user_interface::inform(string const & line)
 {
   string prefixedLine;
-  prefix_lines_with(_("monotone: "), line, prefixedLine);
+  prefix_lines_with(output_prefix(), line, prefixedLine);
   ensure_clean_line();
   clog << sanitize(prefixedLine) << endl;
   clog.flush();
@@ -401,3 +533,11 @@ get_user_locale()
     }
   return user_locale;
 }
+
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
