@@ -1,8 +1,11 @@
-// copyright (C) 2002, 2003 graydon hoare <graydon@pobox.com>
-// copyright (C) 2004 Nathaniel Smith <njs@pobox.com>
-// all rights reserved.
-// licensed to the public under the terms of the GNU GPL (>= 2)
-// see the file COPYING for details
+// Copyright (C) 2002 Graydon Hoare <graydon@pobox.com>
+//
+// This program is made available under the GNU GPL version 2.0 or
+// greater. See the accompanying file COPYING for details.
+//
+// This program is distributed WITHOUT ANY WARRANTY; without even the
+// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+// PURPOSE.
 
 #include <set>
 #include <map>
@@ -11,13 +14,13 @@
 
 #include "app_state.hh"
 #include "database.hh"
-#include "manifest.hh"
 #include "sanity.hh"
 #include "cert.hh"
 #include "transforms.hh"
 #include "ui.hh"
 #include "update.hh"
 #include "vocab.hh"
+#include "revision.hh"
 
 // these functions just encapsulate the (somewhat complex) logic behind
 // picking an update target. the actual updating takes place in
@@ -39,11 +42,14 @@
 // it may be somewhat inefficient to use erase_ancestors here, but deal with
 // that when and if the time comes...
 
+using std::make_pair;
+using std::map;
+using std::set;
+using std::vector;
+
 using boost::lexical_cast;
 
-using namespace std;
-
-static void 
+static void
 get_test_results_for_revision(revision_id const & id,
                               map<rsa_keypair_id, bool> & results,
                               app_state & app)
@@ -56,14 +62,14 @@ get_test_results_for_revision(revision_id const & id,
     {
       cert_value cv;
       decode_base64(i->inner().value, cv);
-      try 
+      try
         {
           bool test_ok = lexical_cast<bool>(cv());
           results.insert(make_pair(i->inner().key, test_ok));
         }
-      catch(boost::bad_lexical_cast & e)
+      catch(boost::bad_lexical_cast &)
         {
-          W(F("failed to decode boolean testresult cert value '%s'\n") % cv);
+          W(F("failed to decode boolean testresult cert value '%s'") % cv);
         }
     }
 }
@@ -75,8 +81,8 @@ acceptable_descendent(cert_value const & branch,
                       revision_id const & target,
                       app_state & app)
 {
-  L(F("Considering update target %s\n") % target);
-  
+  L(FL("Considering update target %s") % target);
+
   // step 1: check the branch
   base64<cert_value> val;
   encode_base64(branch, val);
@@ -85,25 +91,25 @@ acceptable_descendent(cert_value const & branch,
   erase_bogus_certs(certs, app);
   if (certs.empty())
     {
-      L(F("%s not in branch %s\n") % target % branch);
+      L(FL("%s not in branch %s") % target % branch);
       return false;
     }
-  
+
   // step 2: check the testresults
   map<rsa_keypair_id, bool> target_results;
   get_test_results_for_revision(target, target_results, app);
   if (app.lua.hook_accept_testresult_change(base_results, target_results))
     {
-      L(F("%s is acceptable update candidate\n") % target);
+      L(FL("%s is acceptable update candidate") % target);
       return true;
     }
   else
     {
-      L(F("%s has unacceptable test results\n") % target);
+      L(FL("%s has unacceptable test results") % target);
       return false;
     }
 }
-      
+
 
 static void
 calculate_update_set(revision_id const & base,
@@ -128,7 +134,7 @@ calculate_update_set(revision_id const & base,
 
   app.db.get_revision_children(base, children);
   copy(children.begin(), children.end(), back_inserter(to_traverse));
-  
+
   while (!to_traverse.empty())
     {
       revision_id target = to_traverse.back();
@@ -150,8 +156,8 @@ calculate_update_set(revision_id const & base,
 
   erase_ancestors(candidates, app);
 }
-  
-  
+
+
 void pick_update_candidates(revision_id const & base_ident,
                             app_state & app,
                             set<revision_id> & candidates)
@@ -163,5 +169,13 @@ void pick_update_candidates(revision_id const & base_ident,
   calculate_update_set(base_ident, cert_value(app.branch_name()),
                        app, candidates);
 }
-  
 
+
+
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
