@@ -8,6 +8,7 @@
 #include <errno.h>
 #include <windows.h>
 #include <shlobj.h>
+#include <direct.h>
 
 #include <boost/filesystem/path.hpp>
 #include <boost/filesystem/operations.hpp>
@@ -23,7 +24,10 @@ get_current_working_dir()
   char buffer[4096];
   E(getcwd(buffer, 4096),
     F("cannot get working directory: %s") % strerror(errno));
-  return std::string(buffer);
+  std::string str(buffer);
+  if (str[str.size() - 1] == '\\')
+    str = str.substr(0, str.size() - 1);
+  return str;
 }
 
 void
@@ -67,7 +71,7 @@ get_homedir()
   home = getenv("HOME");
   if (home != NULL)
     {
-      L(F("Home directory from HOME\n"));
+      L(FL("Home directory from HOME\n"));
       return std::string(home);
     }
   // Otherwise, try USERPROFILE.  We could also use SHGetFolderPath() to get
@@ -76,7 +80,7 @@ get_homedir()
   char * userprofile = getenv("USERPROFILE");
   if (userprofile != NULL)
     {
-      L(F("Home directory from USERPROFILE\n"));
+      L(FL("Home directory from USERPROFILE\n"));
       return std::string(userprofile);
     }
   // Try concatenating HOMEDRIVE and HOMEPATH
@@ -84,13 +88,13 @@ get_homedir()
   char * homepath = getenv("HOMEPATH");
   if (homedrive != NULL && homepath != NULL)
     {
-      L(F("Home directory from HOMEDRIVE+HOMEPATH\n"));
+      L(FL("Home directory from HOMEDRIVE+HOMEPATH\n"));
       return std::string(homedrive) + std::string(homepath);
     }
   char * systemdrive = getenv("SystemDrive");
   if (systemdrive != NULL)
     {
-      L(F("Home directory from SystemDrive\n"));
+      L(FL("Home directory from SystemDrive\n"));
       return std::string(systemdrive);
     }
   return std::string("C:");
@@ -137,19 +141,28 @@ rename_clobberingly_impl(const char* from, const char* to)
 {
   // MoveFileEx is only available on NT-based systems.  We will revert to a
   // more compatible DeleteFile/MoveFile pair as a compatibility fall-back.
-  typedef BOOL (*MoveFileExFun)(LPCTSTR, LPCTSTR, DWORD);
-  static MoveFileExFun MoveFileEx = 0;
-  if (MoveFileEx == 0) {
+  typedef BOOL (WINAPI *MoveFileExFun)(LPCTSTR, LPCTSTR, DWORD);
+  static MoveFileExFun fnMoveFileEx = 0;
+  static bool MoveFileExAvailable = false;
+  if (fnMoveFileEx == 0) {
     HMODULE hModule = LoadLibrary("kernel32");
-    MoveFileEx = reinterpret_cast<MoveFileExFun>
-      (GetProcAddress(hModule, "MoveFileExA"));
-    if (MoveFileEx)
-      L(F("using MoveFileEx for renames"));
+    if (hModule)
+      fnMoveFileEx = reinterpret_cast<MoveFileExFun>
+        (GetProcAddress(hModule, "MoveFileExA"));
+    if (fnMoveFileEx) {
+      L(FL("using MoveFileEx for renames"));
+      MoveFileExAvailable = true;
+    } else
+      L(FL("using DeleteFile/MoveFile fallback for renames"));
   }
 
-  if (MoveFileEx) {
-    if (MoveFileEx(from, to, MOVEFILE_REPLACE_EXISTING))
+  if (MoveFileExAvailable) {
+    if (fnMoveFileEx(from, to, MOVEFILE_REPLACE_EXISTING))
       return true;
+    else if (GetLastError() == ERROR_CALL_NOT_IMPLEMENTED) {
+      MoveFileExAvailable = false;
+      L(FL("MoveFileEx failed with CALL_NOT_IMPLEMENTED, using fallback"));
+    }
   } else {
     // This is not even remotely atomic, but what can you do?
     DeleteFile(to);
@@ -162,23 +175,25 @@ rename_clobberingly_impl(const char* from, const char* to)
 void
 rename_clobberingly(any_path const & from, any_path const & to)
 {
-  const char* szFrom = from.as_external().c_str();
-  const char* szTo = to.as_external().c_str();
   static const int renameAttempts = 16;
   DWORD sleepTime = 1;
+  DWORD lastError = 0;
 
   // If a clobbering rename attempt fails, we wait and try again, up to an
   // (arbitrary) maximum of 16 attempts.  This is a gross hack to work
   // around the common problem where another process (e.g. a virus checker)
   // will exclusive open a file you've just touched.
   for (int i = 0; i < renameAttempts; ++i) {
-    if (rename_clobberingly_impl(szFrom, szTo))
+    if (rename_clobberingly_impl(from.as_external().c_str(), to.as_external().c_str()))
       return;
-    L(F("attempted rename of '%s' to '%s' failed: %d")
-      % szFrom % szTo % GetLastError());
+    lastError = GetLastError();
+    L(FL("attempted rename of '%s' to '%s' failed: (%s) %d")
+      % from % to % os_strerror(lastError) % lastError);
     Sleep(sleepTime);
     if (sleepTime < 250)
       sleepTime *= 2;
   }
-  E(false, F("renaming '%s' to '%s' failed: %d") % from % to);
+  E(false, F("renaming '%s' to '%s' failed: %s (%d)") % from % to
+           % os_strerror(lastError) % lastError);
 }
+
