@@ -1,7 +1,11 @@
-// copyright (C) 2002, 2003 graydon hoare <graydon@pobox.com>
-// all rights reserved.
-// licensed to the public under the terms of the GNU GPL (>= 2)
-// see the file COPYING for details
+// Copyright (C) 2002 Graydon Hoare <graydon@pobox.com>
+//
+// This program is made available under the GNU GPL version 2.0 or
+// greater. See the accompanying file COPYING for details.
+//
+// This program is distributed WITHOUT ANY WARRANTY; without even the
+// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+// PURPOSE.
 
 // this file implements the xdelta algorithm, produces and consumes simple
 // copy/insert binary patches with the following structure:
@@ -35,13 +39,24 @@
 #include "sanity.hh"
 #include "xdelta.hh"
 
-using namespace std;
+using std::make_pair;
+using std::min;
+using std::ostream;
+using std::ostringstream;
+using std::pair;
+using std::set;
+using std::string;
+using std::vector;
+using std::memcmp;
+using std::lower_bound;
+
+using boost::shared_ptr;
 
 struct identity {size_t operator()(u32 const & v) const { return static_cast<size_t>(v);}};
 typedef pair<string::size_type, string::size_type> extent;
 typedef hashmap::hash_map<u32, extent> match_table;
 
-struct 
+struct
 insn
 {
   insn(char c) : code(insert), pos(0), len(0), payload("")  { payload += c; }
@@ -66,7 +81,7 @@ ostream & operator<<(ostream & ost, insn const & i)
 }
 
 
-static inline void 
+static inline void
 init_match_table(string const & a,
                  string::size_type blocksz,
                  match_table & tab)
@@ -78,19 +93,19 @@ init_match_table(string const & a,
       u32 sum = adler32(reinterpret_cast<u8 const *>(a.data() + i), step).sum();
       if (tab.find(sum) == tab.end())
         tab.insert(make_pair(sum, make_pair(i, step)));
-    }  
+    }
   return;
 }
 
 
 static inline bool
-find_match(match_table const & matches, 
+find_match(match_table const & matches,
            vector<insn> & delta,
-           adler32 const & rolling, 
-           string const & a, 
-           string const & b, 
-           string::size_type bpos, 
-           string::size_type & apos, 
+           adler32 const & rolling,
+           string const & a,
+           string const & b,
+           string::size_type bpos,
+           string::size_type & apos,
            string::size_type & alen,
            string::size_type & badvance)
 {
@@ -103,33 +118,38 @@ find_match(match_table const & matches,
 
   string::size_type tpos = e->second.first;
   string::size_type tlen = e->second.second;
-  
+
   I(tpos < a.size());
   I(tpos + tlen <= a.size());
 
   // maybe it's a false match?
   if (memcmp(a.data() + tpos, b.data() + bpos, tlen) != 0)
     return false;
-  
+
   apos = tpos;
-  alen = tlen;
-  badvance = tlen;
 
   // see if we can extend our match forwards
-  while((apos + alen >= 0)
-        && (bpos + badvance >= 0)
-        && (apos + alen < a.size())
-        && (bpos + badvance < b.size())
-        && (a[apos + alen] == b[bpos + badvance]))
+  string::const_iterator ai = a.begin() + apos + tlen;
+  string::const_iterator ae = a.end();
+  string::const_iterator bi = b.begin() + bpos + tlen;
+  string::const_iterator be = b.end();
+
+  while((ai != ae)
+        && (bi != be)
+        && (*ai == *bi))
     {
-      ++alen;
-      ++badvance;
+      ++tlen;
+      ++ai;
+      ++bi;
     }
 
+  alen = tlen;
+  badvance = tlen;
+
   // see if we can extend backwards into a previous insert hunk
   if (! delta.empty() && delta.back().code == insn::insert)
     {
-      while(apos > 0 
+      while(apos > 0
             && bpos > 0
             && a[apos - 1] == b[bpos - 1]
             && !delta.back().payload.empty())
@@ -144,19 +164,19 @@ find_match(match_table const & matches,
           // 'badvance' forward, just alen.
         }
 
-      // if we've extended back to consume the *entire* insert, 
+      // if we've extended back to consume the *entire* insert,
       // let's do away with it altogether.
       if (delta.back().payload.empty())
         {
           delta.pop_back();
         }
     }
-  
+
   I(memcmp(a.data() + apos, b.data() + bpos, alen) == 0);
   return true;
 }
 
-static inline void 
+static inline void
 insert_insn(vector<insn> & delta, char c)
 {
   if (delta.empty() || delta.back().code == insn::copy)
@@ -166,20 +186,20 @@ insert_insn(vector<insn> & delta, char c)
 }
 
 
-static inline void 
+static inline void
 copy_insn(vector<insn> & delta, string::size_type i, string::size_type matchlen)
 {
   delta.push_back(insn(i, matchlen));
 }
-                      
-  
-static void 
+
+
+static void
 compute_delta_insns(string const & a,
                     string const & b,
                     vector<insn> & delta)
 {
   string::size_type blocksz = 64;
-  match_table matches ((a.size() / blocksz) * 2);
+  match_table matches;
   init_match_table(a, blocksz, matches);
 
   if (b.size() < blocksz)
@@ -188,13 +208,13 @@ compute_delta_insns(string const & a,
         insert_insn(delta, b[i]);
       return;
     }
-  
-  adler32 rolling(reinterpret_cast<u8 const *>(b.data()), blocksz);  
 
-  for (string::size_type 
+  adler32 rolling(reinterpret_cast<u8 const *>(b.data()), blocksz);
+
+  for (string::size_type
        sz = b.size(),
-       lo = 0, 
-       hi = blocksz; 
+       lo = 0,
+       hi = blocksz;
        lo < sz; )
     {
       string::size_type apos = 0, alen = 1, badvance = 1;
@@ -206,11 +226,10 @@ compute_delta_insns(string const & a,
           copy_insn(delta, apos, alen);
         }
       else
-        { 
+        {
           I(apos + alen <= a.size());
           I(alen == 1);
           I(alen < blocksz);
-          I(lo >= 0);
           I(lo < b.size());
           insert_insn(delta, b[lo]);
         }
@@ -218,7 +237,6 @@ compute_delta_insns(string const & a,
       string::size_type next = lo;
       for (; next < lo + badvance; ++next)
         {
-          I(next >= 0);
           I(next < b.size());
           rolling.out(static_cast<u8>(b[next]));
           if (next + blocksz < b.size())
@@ -229,92 +247,21 @@ compute_delta_insns(string const & a,
     }
 }
 
-
-// specialized form for manifest maps, which
-// are sorted, so have far more structure
-
-static void 
-flush_copy(ostringstream & oss, u32 & pos, u32 & len)
-{
-  if (len != 0)
-    {
-      // flush any copy which is going on
-      oss << insn(pos, len);
-      pos = pos + len;
-      len = 0;
-    }
-}
-
-void 
-compute_delta(manifest_map const & a,
-              manifest_map const & b,
-              string & delta)
+void
+write_delta_insns(vector<insn> const & delta_insns,
+		  string & delta)
 {
   delta.clear();
   ostringstream oss;
-
-  manifest_map::const_iterator i = a.begin();
-  manifest_map::const_iterator j = b.begin();
-
-  u32 pos = 0;
-  u32 len = 0;
-  while (j != b.end())
-    {      
-      size_t isz = 0;
-
-      if (i != a.end())
-          isz = i->first.as_internal().size() + 2 + i->second.inner()().size() + 1;
-
-      if (i != a.end() && i->first == j->first)
-        {
-          if (i->second == j->second)
-            {
-              // this line was copied
-              len += isz;
-            }
-          else
-            {
-              // this line was changed, but the *entry* remained, so we
-              // treat it as a simultaneous delete + insert. that means
-              // advance pos over what used to be here, set len to 0, and
-              // copy the new data.
-              flush_copy(oss, pos, len);
-              pos += isz;
-              ostringstream ss;
-              ss << *j;
-              oss << insn(ss.str());              
-            }
-          ++i; ++j;
-        }
-      else         
-        {
-
-          flush_copy(oss, pos, len);
-          
-          if (i != a.end() && i->first < j->first)
-            {              
-              // this line was deleted              
-              ++i;
-              pos += isz;
-            }
-          
-          else
-            {
-              // this line was added
-              ostringstream ss;
-              ss << *j;
-              oss << insn(ss.str());
-              ++j;
-            }
-        }
+  for (vector<insn>::const_iterator i = delta_insns.begin();
+       i != delta_insns.end(); ++i)
+    {
+      oss << *i;
     }
-
-  flush_copy(oss,pos,len);
   delta = oss.str();
 }
 
-
-void 
+void
 compute_delta(string const & a,
               string const & b,
               string & delta)
@@ -341,14 +288,14 @@ compute_delta(string const & a,
     {
       I(a.size() > 0);
       I(b.size() > 0);
-      
-      L(F("computing binary delta instructions\n"));
+
+      L(FL("computing binary delta instructions"));
       compute_delta_insns(a, b, delta_insns);
-      L(F("computed binary delta instructions\n"));
+      L(FL("computed binary delta instructions"));
     }
 
-  ostringstream oss;  
-  for (vector<insn>::const_iterator i = delta_insns.begin(); 
+  ostringstream oss;
+  for (vector<insn>::const_iterator i = delta_insns.begin();
        i != delta_insns.end(); ++i)
     {
       oss << *i;
@@ -356,77 +303,96 @@ compute_delta(string const & a,
   delta = oss.str();
 }
 
-struct 
-simple_applicator 
+struct
+simple_applicator
   : public delta_applicator
 {
   string src;
   string dst;
   virtual ~simple_applicator () {}
-  virtual void begin(string const & base) 
-  {     
-    src = base; 
+  virtual void begin(string const & base)
+  {
+    src = base;
     dst.clear();
   }
-  virtual void next() 
-  { 
-    swap(src,dst); 
-    dst.clear(); 
+  virtual void next()
+  {
+    swap(src,dst);
+    dst.clear();
   }
-  virtual void finish(string & out) 
-  { 
-    out = src; 
+  virtual void finish(string & out)
+  {
+    out = src;
   }
 
-  virtual void copy(string::size_type pos, string::size_type len) 
-  { 
-    dst.append(src, pos, len); 
+  virtual void copy(string::size_type pos, string::size_type len)
+  {
+    dst.append(src, pos, len);
   }
   virtual void insert(string const & str)
-  { 
-    dst.append(str); 
+  {
+    dst.append(str);
   }
 };
 
-void 
-apply_delta(boost::shared_ptr<delta_applicator> da,
-            std::string const & delta)
+inline string::size_type
+read_num(string::const_iterator &i,
+         string::const_iterator e)
 {
-  istringstream del(delta);
-  for (char c = del.get(); c == 'I' || c == 'C'; c = del.get())
+  string::size_type n = 0;
+
+  while (i != e && *i == ' ')
+    ++i;
+
+  while (i != e && *i >= '0' && *i <= '9')
     {
-      I(del.good());
-      if (c == 'I')
-        { 
-          string::size_type len = string::npos;
-          del >> len;
-          I(del.good());
-          I(len != string::npos);
-          string tmp;
-          tmp.reserve(len);
-          I(del.get(c).good());
-          I(c == '\n');
-          while(len--)
-            {
-              I(del.get(c).good());
-              tmp += c;
-            }
-          I(del.get(c).good());
-          I(c == '\n');
-          da->insert(tmp);
+      n *= 10;
+      n += static_cast<size_t>(*i - '0');
+      ++i;
+    }
+  return n;
+}
+
+void
+apply_delta(shared_ptr<delta_applicator> da,
+            string const & delta)
+{
+  string::const_iterator i = delta.begin();
+  while (i != delta.end() && (*i == 'I' || *i == 'C'))
+    {
+      if (*i == 'I')
+        {
+          ++i;
+          I(i != delta.end());
+          string::size_type len = read_num(i, delta.end());
+          I(i != delta.end());
+          I(*i == '\n');
+          ++i;
+          I(i != delta.end());
+	  I((i - delta.begin()) + len <= delta.size());
+	  if (len > 0)
+	    {
+	      string tmp(i, i+len);
+	      da->insert(tmp);
+	    }
+	  i += len;
         }
       else
         {
-          string::size_type pos = string::npos, len = string::npos;
-          del >> pos >> len;          
-          I(del.good());
-          I(len != string::npos);
-          I(del.get(c).good());
-          I(c == '\n');
-          da->copy(pos, len);
+          I(*i == 'C');
+          ++i;
+          I(i != delta.end());
+          string::size_type pos = read_num(i, delta.end());
+          I(i != delta.end());
+          string::size_type len = read_num(i, delta.end());
+	  if (len != 0)
+	    da->copy(pos, len);
         }
-    }    
-  I(del.eof());
+      I(i != delta.end());
+      I(*i == '\n');
+      ++i;
+    }
+  I(i == delta.end());
 }
 
 void
@@ -434,36 +400,36 @@ apply_delta(string const & a,
             string const & delta,
             string & b)
 {
-  boost::shared_ptr<delta_applicator> da(new simple_applicator());
+  shared_ptr<delta_applicator> da(new simple_applicator());
   da->begin(a);
   apply_delta(da, delta);
   da->next();
   da->finish(b);
 }
 
-struct 
+struct
 size_accumulating_delta_applicator :
   public delta_applicator
 {
   u64 & sz;
   size_accumulating_delta_applicator(u64 & s) : sz(s) {}
-  virtual void begin(std::string const & base) {}
+  virtual void begin(string const & base) {}
   virtual void next() {}
-  virtual void finish(std::string & out) {}
+  virtual void finish(string & out) {}
 
-  virtual void copy(std::string::size_type pos, 
-                    std::string::size_type len) 
+  virtual void copy(string::size_type pos,
+                    string::size_type len)
   { sz += len; }
-  virtual void insert(std::string const & str) 
+  virtual void insert(string const & str)
   { sz += str.size(); }
 };
 
 
-u64 
-measure_delta_target_size(std::string const & delta)
+u64
+measure_delta_target_size(string const & delta)
 {
   u64 sz = 0;
-  boost::shared_ptr<delta_applicator> da(new size_accumulating_delta_applicator(sz));
+  shared_ptr<delta_applicator> da(new size_accumulating_delta_applicator(sz));
   apply_delta(da, delta);
   return sz;
 }
@@ -478,7 +444,7 @@ typedef string::size_type length;
 typedef unsigned long piece_id;
 
 struct chunk
-{  
+{
   length len;        // how many chars in this chunk
   piece_id piece;    // which piece to take chars from
   version_pos vpos;  // position in the current version
@@ -505,25 +471,24 @@ struct chunk
 
 typedef vector<chunk> version_spec;
 
-struct 
+struct
 piece_table
 {
   vector<string> pieces;
 
-  void clear() 
+  void clear()
   {
     pieces.clear();
   }
 
-  piece_id insert(string const & p) 
-  { 
-    pieces.push_back(p); 
-    return pieces.size() - 1; 
+  piece_id insert(string const & p)
+  {
+    pieces.push_back(p);
+    return pieces.size() - 1;
   }
 
   void append(string & targ, piece_id p, piece_pos pp, length ln)
   {
-    I(p >= 0);
     I(p < pieces.size());
     targ.append(pieces[p], pp, ln);
   }
@@ -533,36 +498,36 @@ piece_table
     out.clear();
     for (version_spec::const_iterator i = in.begin();
          i != in.end(); ++i)
-      {        
+      {
         append(out, i->piece, i->ppos, i->len);
       }
   }
 };
 
-static void 
+static void
 apply_insert(piece_table & p, version_spec & out, string const & str)
 {
   piece_id piece = p.insert(str);
   version_pos vpos = 0;
-  if (!out.empty()) 
+  if (!out.empty())
     vpos = out.back().vpos + out.back().len;
   out.push_back(chunk(str.size(), piece, vpos, 0));
 }
 
-struct 
+struct
 chunk_less_than
 {
-  bool operator()(chunk const & ch, version_pos vp) const
+  bool operator()(chunk const & ch1, chunk const & ch2) const
   {
     // nb: ch.vpos + ch.len is the 0-based index of the first element *not*
     // included in ch; thus we measure against ch.len - 1.
-    I(ch.len > 0);
-    return (ch.vpos + ch.len - 1) < vp;
+//    I(ch1.len > 0);
+    return (ch1.vpos + ch1.len - 1) < ch2.vpos;
   }
 };
 
-static void 
-apply_copy(version_spec const & in, version_spec & out, 
+static void
+apply_copy(version_spec const & in, version_spec & out,
            version_pos src_vpos, length src_len)
 {
   //
@@ -591,9 +556,10 @@ apply_copy(version_spec const & in, version_spec & out,
   if (!out.empty())
     dst_vpos = out.back().vpos + out.back().len;
   version_pos dst_final = dst_vpos + src_len;
-  version_spec::const_iterator lo = lower_bound(in.begin(), 
-                                                in.end(), 
-                                                src_vpos, 
+  chunk src_bounding_chunk(0,0,src_vpos,0);
+  version_spec::const_iterator lo = lower_bound(in.begin(),
+                                                in.end(),
+                                                src_bounding_chunk,
                                                 chunk_less_than());
   for ( ; src_len > 0; ++lo)
     {
@@ -628,8 +594,8 @@ apply_copy(version_spec const & in, version_spec & out,
       out.push_back(lo->subchunk(dst_vpos, seglen, offset));
       src_vpos += seglen;
       dst_vpos += seglen;
+      I(src_len >= seglen);
       src_len -= seglen;
-      I(src_len >= 0);
       I(out.back().vpos + out.back().len == dst_vpos);
     }
 
@@ -639,13 +605,13 @@ apply_copy(version_spec const & in, version_spec & out,
 }
 
 
-struct 
-piecewise_applicator 
+struct
+piecewise_applicator
   : public delta_applicator
 {
   piece_table pt;
-  boost::shared_ptr<version_spec> src;
-  boost::shared_ptr<version_spec> dst;
+  shared_ptr<version_spec> src;
+  shared_ptr<version_spec> dst;
 
   piecewise_applicator() :
     src(new version_spec()),
@@ -654,53 +620,165 @@ piecewise_applicator
 
   virtual ~piecewise_applicator () {}
 
-  virtual void begin(string const & base) 
-  { 
+  virtual void begin(string const & base)
+  {
     pt.clear();
     piece_id piece = pt.insert(base);
     src->clear();
     src->push_back(chunk(base.size(), piece, 0, 0));
-    dst->clear();    
+    dst->clear();
   }
 
-  virtual void next() 
-  { 
-    swap(src,dst); 
-    dst->clear(); 
+  virtual void next()
+  {
+    swap(src,dst);
+    dst->clear();
   }
 
-  virtual void finish(string & out) 
-  { 
+  virtual void finish(string & out)
+  {
     out.clear();
     pt.build(*src, out);
   }
 
-  virtual void copy(string::size_type pos, string::size_type len) 
-  { 
+  virtual void copy(string::size_type pos, string::size_type len)
+  {
     apply_copy(*src, *dst, pos, len);
   }
 
   virtual void insert(string const & str)
-  { 
+  {
     apply_insert(pt, *dst, str);
   }
 };
 
 
-// these just hide our implementation types from outside 
+// these just hide our implementation types from outside
 
-boost::shared_ptr<delta_applicator> 
+shared_ptr<delta_applicator>
 new_simple_applicator()
 {
-  return boost::shared_ptr<delta_applicator>(new simple_applicator());
+  return shared_ptr<delta_applicator>(new simple_applicator());
 }
 
-boost::shared_ptr<delta_applicator> 
+shared_ptr<delta_applicator>
 new_piecewise_applicator()
 {
-  return boost::shared_ptr<delta_applicator>(new piecewise_applicator());
+  return shared_ptr<delta_applicator>(new piecewise_applicator());
 }
 
+
+// inversion
+
+struct copied_extent
+{
+  copied_extent(string::size_type op,
+		string::size_type np,
+		string::size_type len)
+    : old_pos(op),
+      new_pos(np),
+      len(len)
+  {}
+  string::size_type old_pos;
+  string::size_type new_pos;
+  string::size_type len;
+  bool operator<(copied_extent const & other) const
+  {
+    return (old_pos < other.old_pos) ||
+      (old_pos == other.old_pos && len > other.len);
+  }
+};
+
+struct
+inverse_delta_writing_applicator :
+  public delta_applicator
+{
+  string const & old;
+  set<copied_extent> copied_extents;
+  string::size_type new_pos;
+
+  inverse_delta_writing_applicator(string const & old)
+    : old(old),
+      new_pos(0)
+  {}
+
+  virtual void begin(string const & base) {}
+  virtual void next() {}
+  virtual void finish(string & out)
+  {
+    // We are trying to write a delta instruction stream which
+    // produces 'old' from 'new'. We don't care what was in 'new',
+    // because we're only going to copy some parts forwards, and we
+    // already know which parts: those in the table. Our table lists
+    // extents which were copied in order that they appear in 'old'.
+    //
+    // When we run into a section of 'old' which isn't in the table,
+    // we have to emit an insert instruction for the gap.
+
+    string::size_type old_pos = 0;
+    out.clear();
+    vector<insn> delta_insns;
+
+    for (set<copied_extent>::iterator i = copied_extents.begin();
+	 i != copied_extents.end(); ++i)
+      {	
+	// It is possible that this extent left a gap after the
+	// previously copied extent; in this case we wish to pad
+	// the intermediate space with an insert.
+	while (old_pos < i->old_pos)
+	  {
+	    I(old_pos < old.size());
+	    // Don't worry, adjacent inserts are merged.
+	    insert_insn(delta_insns, old.at(old_pos++));
+	  }
+
+	// It is also possible that this extent *overlapped* the
+	// previously copied extent; in this case we wish to subtract
+	// the overlap from the inverse copy.
+
+	string::size_type overlap = 0;
+	if (i->old_pos < old_pos)
+	  overlap = old_pos - i->old_pos;
+
+	if (i->len <= overlap)
+	  continue;
+
+	I(i->len > overlap);
+	copy_insn(delta_insns, i->new_pos + overlap, i->len - overlap);
+	old_pos += (i->len - overlap);
+      }
+    while (old_pos < old.size())
+      insert_insn(delta_insns, old.at(old_pos++));
+
+    write_delta_insns(delta_insns, out);
+  }
+
+  virtual void copy(string::size_type old_pos,
+                    string::size_type len)
+  {
+    I(old_pos < old.size());
+    copied_extents.insert(copied_extent(old_pos, new_pos, len));
+    new_pos += len;
+  }
+
+  virtual void insert(string const & str)
+  {
+    new_pos += str.size();
+  }
+};
+
+
+void
+invert_xdelta(string const & old_str,
+	      string const & delta,
+	      string & delta_inverse)
+{
+  shared_ptr<delta_applicator> da(new inverse_delta_writing_applicator(old_str));
+  apply_delta(da, delta);
+  da->finish(delta_inverse);
+}
+
+
 #ifdef BUILD_UNIT_TESTS
 
 #include "unit_tests.hh"
@@ -722,22 +800,22 @@ boost::uniform_smallint<boost::mt19937, char> xdelta_chargen(xdelta_prng, 'a', '
 boost::uniform_smallint<boost::mt19937, size_t> xdelta_sizegen(xdelta_prng, 1024, 65536);
 boost::uniform_smallint<boost::mt19937, size_t> xdelta_editgen(xdelta_prng, 3, 10);
 boost::uniform_smallint<boost::mt19937, size_t> xdelta_lengen(xdelta_prng, 1, 256);
-#define PRNG 
+#define PRNG
 #endif
 
-void 
+void
 xdelta_random_string(string & str)
 {
   size_t sz = xdelta_sizegen(PRNG);
   str.clear();
-  str.reserve(sz);   
+  str.reserve(sz);
   while(sz-- > 0)
     {
       str += xdelta_chargen(PRNG);
     }
 }
 
-void 
+void
 xdelta_randomly_insert(string & str)
 {
   size_t nedits = xdelta_editgen(PRNG);
@@ -756,7 +834,7 @@ xdelta_randomly_insert(string & str)
     }
 }
 
-void 
+void
 xdelta_randomly_change(string & str)
 {
   size_t nedits = xdelta_editgen(PRNG);
@@ -772,7 +850,7 @@ xdelta_randomly_change(string & str)
     }
 }
 
-void 
+void
 xdelta_randomly_delete(string & str)
 {
   size_t nedits = xdelta_editgen(PRNG);
@@ -787,7 +865,7 @@ xdelta_randomly_delete(string & str)
     }
 }
 
-void 
+void
 xdelta_random_simple_delta_test()
 {
   for (int i = 0; i < 100; ++i)
@@ -800,26 +878,26 @@ xdelta_random_simple_delta_test()
       xdelta_randomly_delete(b);
       compute_delta(a, b, fdel);
       compute_delta(b, a, rdel);
-      L(boost::format("src %d, dst %d, fdel %d, rdel %d\n")
+      L(FL("src %d, dst %d, fdel %d, rdel %d")
         % a.size() % b.size()% fdel.size() % rdel.size()) ;
       if (fdel.size() == 0)
         {
-          L(boost::format("confirming src == dst and rdel == 0\n"));
+          L(FL("confirming src == dst and rdel == 0"));
           BOOST_CHECK(a == b);
           BOOST_CHECK(rdel.size() == 0);
-        }      
+        }
       else
         {
           apply_delta(a, fdel, c);
           apply_delta(b, rdel, d);
-          L(boost::format("confirming dst1 %d, dst2 %d\n") % c.size() % d.size());
+          L(FL("confirming dst1 %d, dst2 %d") % c.size() % d.size());
           BOOST_CHECK(b == c);
           BOOST_CHECK(a == d);
         }
     }
 }
 
-void 
+void
 add_xdelta_tests(test_suite * suite)
 {
   I(suite);
@@ -827,3 +905,11 @@ add_xdelta_tests(test_suite * suite)
 }
 
 #endif // BUILD_UNIT_TESTS
+
+// Local Variables:
+// mode: C++
+// fill-column: 76
+// c-file-style: "gnu"
+// indent-tabs-mode: nil
+// End:
+// vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
